<?php
namespace Rtrs\Controllers\Admin\Meta;
use Rtrs\Helpers\Functions; 

class SingleMetaOptions { 

    private $prefix = "rtrs_";

    /**
     * Marge all meta field
     *
     * @return array
     */
    function allMetaFields() { 
        $fields = array();
        $fieldsA = array_merge( 
            $this->sectionSchemaFields(), 
            $this->sectionReviewFields(), 
        );
        foreach ($fieldsA as $field) {
            $fields[] = $field;
        } 

        return $fields;
    } 

    function sectionReviewFields() { 
        $post_type = get_post_type();
        $review_fields = array(
            array(
                "type"  => "info",
                "label" => esc_html__("Total rating", 'review-schema'),
                "doc" => esc_html__("Total rating of this", 'review-schema') . ' ' . $post_type,
                "data"  => "total_rating", 
            ), 
            array(
                "type"  => "info",
                "label" => esc_html__("Average rating", 'review-schema'),
                "doc" => esc_html__("Average rating of this", 'review-schema') . ' ' . $post_type,
                "data"  => "avg_rating", 
            ),
            array(
                "type"  => "info",
                "label" => esc_html__("Best rating", 'review-schema'),
                "doc" => esc_html__("Best rating of this", 'review-schema') . ' ' . $post_type,
                "data"  => "best_rating", 
            ), 
            array(
                "type"  => "info",
                "label" => esc_html__("Worst rating", 'review-schema'),
                "doc" => esc_html__("Worst rating of this", 'review-schema') . ' ' . $post_type,
                "data"  => "worst_rating", 
            ), 
            /* 
            TODO: do it later
            array(
                "type"  => "info",
                "label" => esc_html__("Total recommended", 'review-schema'),
                "doc" => esc_html__("Total rating of this", 'review-schema') . ' ' . $post_type,
                "data"  => "total_recommended", 
            ),  */
        );
        return apply_filters('rtrs_affiliate_review_fields', $review_fields);
    }

    function sectionSchemaFields() {
        $settings_fields = array( 
            array(
                "type"    => "checkbox",
                "name"    => "_rtrs_custom_rich_snippet", 
                "id"      => "rtrs-rich_snippet",
                "label"   => esc_html__("Custom rich snippet? (Manual)", 'review-schema'), 
                "desc"   => esc_html__("If you enable this option. Schema JSON-LD data will generate from here.", 'review-schema'), 
                "option"  => esc_html__("Enable", 'review-schema')
            ),  
            array(
                "type"    => "select2",
                "name"    => "_rtrs_rich_snippet_cat",
                "label"   => esc_html__("Rich snippet type", 'review-schema'),
                'multiple' => true,
                "id"      => "rtrs-rich_snippet_cat",
                "options" => Functions::rich_snippet_cats()
            ), 
            //article 
            array(
                "type"    => "group",
                "name"    => $this->prefix . "article_schema", 
                "id"      => "rtrs-article_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Article schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),                     
                    array(
                        'name'     => 'headline',
                        'type'     => 'text',
                        'label'    => esc_html__('Headline', "review-schema"),
                        'desc'     => esc_html__('Article title', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'mainEntityOfPage',
                        'type'     => 'url',
                        'label'    => esc_html__('Page URL', "review-schema"),
                        'desc'     => esc_html__('The canonical URL of the article page', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author',
                        'type'     => 'text',
                        'label'    => esc_html__('Author Name', "review-schema"),
                        'desc'     => esc_html__('Author display name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author_url',
                        'type'     => 'text',
                        'label'    => esc_html__('Author URL', "review-schema"), 
                        'required' => true
                    ),
                    array(
                        'name'    => 'image',
                        'type'     => 'image',
                        'label'    => esc_html__('Article Feature Image', "review-schema"),
                        'required' => true,
                        'desc'     => wp_kses( __('Images should be at least 696 pixels wide.<br>Images should be in .jpg, .png, or. gif format.', "review-schema"), ['br' => [] ] )
                    ),
                    array(
                        'name'     => 'datePublished',
                        'type'     => 'text',
                        'label'    => esc_html__('Published date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'required' => true,
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'dateModified',
                        'type'     => 'text',
                        'label'    => esc_html__('Modified date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'required' => true,
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'publisher',
                        'type'     => 'text',
                        'label'    => esc_html__('Publisher', "review-schema"),
                        'desc'     => esc_html__('Publisher name or Organization name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'publisherImage',
                        'type'     => 'image',
                        'label'    => esc_html__('Publisher Logo', "review-schema"),
                        'desc'     => wp_kses( __('Logos should have a wide aspect ratio, not a square icon.<br>Logos should be no wider than 600px, and no taller than 60px.<br>Always retain the original aspect ratio of the logo when resizing. Ideally, logos are exactly 60px tall with width <= 600px. If maintaining a height of 60px would cause the width to exceed 600px, downscale the logo to exactly 600px wide and reduce the height accordingly below 60px to maintain the original aspect ratio.<br>', "review-schema"), ['br' => []]),
                        'required' => true
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                        'desc'  => esc_html__('Short description', "review-schema")
                    ),
                    array(
                        'name'  => 'articleBody',
                        'type'  => 'textarea',
                        'label' => esc_html__('Article body', "review-schema"),
                        'desc'  => esc_html__('Article content', "review-schema")
                    ),
                    array(
                        'name'  => 'alternativeHeadline',
                        'type'  => 'text',
                        'label' => esc_html__('Alternative headline', "review-schema"),
                        'desc'  => esc_html__('A secondary headline for the article.', "review-schema")
                    ),
                ]
            ), 
            //news_article
            array(
                "type"    => "group",
                "name"    => $this->prefix . "news_article_schema", 
                "id"      => "rtrs-news_article_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("News article schema", 'review-schema'),  
                "fields" => [
                    /* array(
                        'name'  => 'news_article',
                        'type'  => 'button',
                        'label' => esc_html__('Regenerate schema', "review-schema"),
                        'value' => esc_html__('Regenerate', "review-schema"),
                        'doc'  => esc_html__("This will auto fill schema data again.", "review-schema")
                    ), */
                    array(
                        "type"      => "tab",
                        "name"      => "status",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'headline',
                        'type'     => 'text',
                        'label'    => esc_html__('Headline', "review-schema"),
                        'desc'     => esc_html__('Article title', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'mainEntityOfPage',
                        'type'     => 'url',
                        'label'    => esc_html__('Page URL', "review-schema"),
                        'desc'     => esc_html__('The canonical URL of the article page', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author',
                        'type'     => 'text',
                        'label'    => esc_html__('Author Name', "review-schema"),
                        'desc'     => esc_html__('Author display name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author_url',
                        'type'     => 'text',
                        'label'    => esc_html__('Author URL', "review-schema"), 
                        'required' => true
                    ),
                    array(
                        'name'    => 'image',
                        'type'     => 'image',
                        'label'    => esc_html__('Article Feature Image', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__('Images should be at least 696 pixels wide.<br>Images should be in .jpg, .png, or. gif format.', "review-schema")
                    ),
                    array(
                        'name'     => 'datePublished',
                        'type'     => 'text',
                        'label'    => esc_html__('Published date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'required' => true,
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'dateModified',
                        'type'     => 'text',
                        'label'    => esc_html__('Modified date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'required' => true,
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'publisher',
                        'type'     => 'text',
                        'label'    => esc_html__('Publisher', "review-schema"),
                        'desc'     => esc_html__('Publisher name or Organization name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'publisherImage',
                        'type'     => 'image',
                        'label'    => esc_html__('Publisher Logo', "review-schema"),
                        'desc'     => wp_kses( __('Logos should have a wide aspect ratio, not a square icon.<br>Logos should be no wider than 600px, and no taller than 60px.<br>Always retain the original aspect ratio of the logo when resizing. Ideally, logos are exactly 60px tall with width <= 600px. If maintaining a height of 60px would cause the width to exceed 600px, downscale the logo to exactly 600px wide and reduce the height accordingly below 60px to maintain the original aspect ratio.<br>', "review-schema"), ['br' => []]),
                        'required' => true
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                        'desc'  => esc_html__('Short description', "review-schema")
                    ),
                    array(
                        'name'  => 'articleBody',
                        'type'  => 'textarea',
                        'label' => esc_html__('Article body', "review-schema"),
                        'desc'  => esc_html__('Article content', "review-schema")
                    ),
                ]
            ), 
            //blog_posting
            array(
                "type"    => "group",
                "name"    => $this->prefix . "blog_posting_schema", 
                "id"      => "rtrs-blog_posting_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Blog posting schema", 'review-schema'),  
                "fields" => [
                    /* array(
                        'name'  => 'blog_posting',
                        'type'  => 'button',
                        'label' => esc_html__('Regenerate schema', "review-schema"),
                        'value' => esc_html__('Regenerate', "review-schema"),
                        'doc'  => esc_html__("This will auto fill schema data again.", "review-schema")
                    ), */
                    array(
                        "type"      => "tab",
                        "name"      => "status",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'headline',
                        'type'     => 'text',
                        'label'    => esc_html__('Headline', "review-schema"),
                        'desc'     => esc_html__('Blog posting title', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'mainEntityOfPage',
                        'type'     => 'url',
                        'label'    => esc_html__('Page URL', "review-schema"),
                        'desc'     => esc_html__('The canonical URL of the article page', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author',
                        'type'     => 'text',
                        'label'    => esc_html__('Author name', "review-schema"),
                        'desc'     => esc_html__('Author display name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author_url',
                        'type'     => 'text',
                        'label'    => esc_html__('Author URL', "review-schema"), 
                        'required' => true
                    ),
                    array(
                        'name'     => 'image',
                        'type'     => 'image',
                        'label'    => esc_html__('Feature Image', "review-schema"),
                        'desc'     => esc_html__("The representative image of the article. Only a marked-up image that directly belongs to the article should be specified.<br> Images should be at least 696 pixels wide. <br>Images should be in .jpg, .png, or. gif format.", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'datePublished',
                        'type'     => 'text',
                        'label'    => esc_html__('Published date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'dateModified',
                        'type'     => 'text',
                        'label'    => esc_html__('Modified date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'publisher',
                        'type'     => 'text',
                        'label'    => esc_html__('Publisher', "review-schema"),
                        'desc'     => esc_html__('Publisher name or Organization name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'publisherImage',
                        'type'     => 'image',
                        'label'    => esc_html__('Publisher Logo', "review-schema"),
                        'desc'     => esc_html__("Logos should have a wide aspect ratio, not a square icon.<br>Logos should be no wider than 600px, and no taller than 60px.<br>Always retain the original aspect ratio of the logo when resizing. Ideally, logos are exactly 60px tall with width <= 600px. If maintaining a height of 60px would cause the width to exceed 600px, downscale the logo to exactly 600px wide and reduce the height accordingly below 60px to maintain the original aspect ratio.<br>", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                        'desc'  => esc_html__('Short description', "review-schema")
                    ),
                    array(
                        'name'  => 'articleBody',
                        'type'  => 'textarea',
                        'label' => esc_html__('Article body', "review-schema"),
                        'desc'  => esc_html__('Article content', "review-schema")
                    )
                ]
            ),
            //event
            array(
                "type"    => "group",
                "name"    => $this->prefix . "event_schema", 
                "id"      => "rtrs-event_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Event schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The name of the event.", "review-schema")
                    ),
                    array(
                        'name'     => 'locationName',
                        'type'     => 'text',
                        'label'    => esc_html__('Location name', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("Event Location name", "review-schema")
                    ),
                    array(
                        'name'     => 'locationAddress',
                        'type'     => 'text',
                        'label'    => esc_html__('Location address', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The location of for example where the event is happening, an organization is located, or where an action takes place.", "review-schema")
                    ),
                    array(
                        'name'     => 'startDate',
                        'type'     => 'text',
                        'label'    => esc_html__('Start date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'required' => true,
                        'desc'     => esc_html__("Event start date Like : 2020-10-16 4:00 AM", "review-schema")
                    ),
                    array(
                        'name'        => 'endDate',
                        'type'        => 'text',
                        'label'       => esc_html__('End date', "review-schema"),
                        'recommended' => true,
                        'class'       => 'rtrs-date',
                        'desc'        => esc_html__("Event end date Like : 2020-10-16 4:00 AM", "review-schema")
                    ),
                    array(
                        'name'        => 'description',
                        'type'        => 'textarea',
                        'label'       => esc_html__('Description', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("Event description", "review-schema")
                    ),
                    array(
                        'name'        => 'performerName',
                        'type'        => 'text',
                        'label'       => esc_html__('Performer Name', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("The performer's name.", "review-schema")
                    ),
                    array(
                        'name'        => 'image',
                        'type'        => 'url',
                        'label'       => esc_html__('Image URL', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("URL of an image or logo for the event or tour", "review-schema")
                    ),
                    array(
                        'name'        => 'price',
                        'type'        => 'number',
                        'label'       => esc_html__('Price', "review-schema"),
                        'recommended' => true,  
                        'desc'  => wp_kses( __("This is highly recommended. The lowest available price, including service charges and fees, of this type of ticket. <span class='required'>Not required but (Recommended)</span>", "review-schema"), [ 'span' => [] ] )
                    ),
                    array(
                        'name'  => 'priceCurrency',
                        'type'  => 'text',
                        'label' => esc_html__('Price currency', "review-schema"),
                        'desc'  => esc_html__("The 3-letter currency code. (USD)", "review-schema")
                    ),
                    array(
                        "name"        => "availability",
                        'type'        => 'select',
                        'label'       => esc_html__("Availability", 'review-schema'),
                        'recommended' => true,
                        'default'       => "http://schema.org/InStock",
                        'options'     => array(
                            'http://schema.org/InStock'  => esc_html__('InStock', 'review-schema'),
                            'http://schema.org/SoldOut'  => esc_html__('SoldOut', 'review-schema'),
                            'http://schema.org/PreOrder' => esc_html__('PreOrder', 'review-schema'),
                        ),
                    ),
                    array(
                        "name"        => "eventStatus",
                        'type'        => 'select',
                        'label'       => esc_html__("Event status", 'review-schema'),
                        'recommended' => true,
                        'default'       => "https://schema.org/EventScheduled",
                        'options'     => array(
                            'https://schema.org/EventCancelled'  => esc_html__('Cancelled', 'review-schema'), 
                            'https://schema.org/EventMovedOnline'  => esc_html__('Moved Online', 'review-schema'), 
                            'https://schema.org/EventPostponed'  => esc_html__('Postponed', 'review-schema'), 
                            'https://schema.org/EventRescheduled'  => esc_html__('Rescheduled', 'review-schema'), 
                            'https://schema.org/EventScheduled'  => esc_html__('Scheduled', 'review-schema'), 
                        ),
                    ),
                    array(
                        "name"        => "eventAttendanceMode",
                        'type'        => 'select',
                        'label'       => esc_html__("Attendance mode", 'review-schema'),
                        'recommended' => true,
                        'default'       => "https://schema.org/OfflineEventAttendanceMode",
                        'options'     => array(
                            'https://schema.org/OfflineEventAttendanceMode'  => esc_html__('Offline', 'review-schema'),
                            'https://schema.org/OnlineEventAttendanceMode'  => esc_html__('Online', 'review-schema'),
                            'https://schema.org/MixedEventAttendanceMode' => esc_html__('Mixed', 'review-schema'),
                        ),
                    ),
                    array(
                        'name'        => 'validFrom',
                        'type'        => 'text',
                        'label'       => esc_html__('Valid From', "review-schema"),
                        'recommended' => true,
                        'class'       => 'rtrs-date',
                        'desc'        => sprintf("The date and time when tickets go on sale (only required on date-restricted offers), in <a href='%s' target='_blank'>ISO-8601 format</a>", 'https://en.wikipedia.org/wiki/ISO_8601')
                    ),
                    array(
                        'name' => 'url',
                        'type'        => 'url',
                        'recommended' => true,
                        'label'       => esc_html__('URL', "review-schema"),
                        'placeholder' => esc_html__('URL', "review-schema"),
                        'desc'  => wp_kses( __("A link to the event's details page. <span class='required'>Not required but (Recommended)</span>", "review-schema"), [ 'span' => [] ] )
                    ),
                    array(
                        'name' => 'organizerName',
                        'type'        => 'text',
                        'recommended' => true,
                        'label'       => esc_html__('Organization Name', "review-schema"), 
                    ),
                    array(
                        'name' => 'organizerUrl',
                        'type'        => 'url',
                        'recommended' => true,
                        'label'       => esc_html__('Organization URL', "review-schema"), 
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ), 
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Review Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder'  => "https://facebook.com/example&#10;https://twitter.com/example",
                        'required' => true,
                        'desc'  => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'  => 'review_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"), 
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"), 
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    ) 
                ]
            ),
            //job_posting
            array(
                "type"    => "group",
                "name"    => $this->prefix . "job_posting_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-job_posting_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Job posting schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        'name'  => 'title', 
                        'type'  => 'text',
                        'label' => esc_html__('Title', "review-schema"),
                    ),
                    array(
                        'name'  => 'salaryAmount',
                        'type'  => 'number',
                        'label' => esc_html__('Base Salary', "review-schema"), 
                        'desc'  => esc_html__('50.00', "review-schema")
                    ),
                    array(
                        'name'  => 'currency',
                        'type'  => 'text',
                        'label' => esc_html__('Currency', "review-schema"),
                        'desc'  => esc_html__('USD', "review-schema")
                    ),
                    array(
                        'name'    => 'salaryAt',
                        'type'    => 'select',
                        'label'   => esc_html__('Salary at', "review-schema"),
                        'options' => array(
                            'MONTH'   => esc_html__('MONTH', "review-schema"),
                            'HOUR'   => esc_html__('HOUR', "review-schema"),
                            'WEEK'   => esc_html__('WEEK', "review-schema"),
                            'YEAR'   => esc_html__('YEAR', "review-schema"),
                        )
                    ),
                    array(
                        'name'  => 'datePosted',
                        'type'  => 'text',
                        'label' => esc_html__('Job posted date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'  => 'validThrough',
                        'type'  => 'text',
                        'label' => esc_html__('Valid date through', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                    ),
                    array(
                        'name'    => 'employmentType',
                        'type'    => 'select',
                        'label'   => esc_html__('Employment Type', "review-schema"),
                        'options' => array(
                            'full-time' => esc_html__('Full Time', "review-schema"),
                            'part-time' => esc_html__('Part Time', "review-schema"),
                            'contract' => esc_html__('Contract', "review-schema"),
                            'temporary' => esc_html__('Temporary', "review-schema"),
                            'seasonal' => esc_html__('Seasonal', "review-schema"),
                            'internship' => esc_html__('Internship', "review-schema"),
                        )
                    ),
                    array(
                        'name'  => 'workHours',
                        'type'  => 'text',
                        'label' => esc_html__('working Hours', "review-schema"),
                        'desc'  => esc_html__("40 hours per week", "review-schema")
                    ),
                    array(
                        'name'  => 'hiringOrganization',
                        'type'  => 'text',
                        'label' => esc_html__('Hiring Organization', "review-schema"),
                    ),
                    array(
                        'name'  => 'addressLocality',
                        'type'  => 'text',
                        'label' => esc_html__('Job location address', "review-schema"),
                        'desc'  => esc_html__('City (i.e Melbourne)', "review-schema")
                    ),
                    array(
                        'name'  => 'addressRegion',
                        'type'  => 'text',
                        'label' => esc_html__('Job location region', "review-schema"),
                        'desc'  => esc_html__('State (i.e. Victoria)', "review-schema")
                    ),
                    array(
                        'name'  => 'postalCode',
                        'type'  => 'text',
                        'label' => esc_html__('Location Postal code', "review-schema"),
                    ),
                    array(
                        'name'  => 'streetAddress',
                        'type'  => 'text',
                        'label' => esc_html__('Location street Address', "review-schema"),
                    ),
                    array(
                        'name'  => 'jobBenefits',
                        'type'  => 'text',
                        'label' => esc_html__('Job Benefits', "review-schema"),
                        'desc'  => esc_html__("Medical, Life, Dental", "review-schema")
                    ),
                    array(
                        'name'  => 'educationRequirements',
                        'type'  => 'text',
                        'label' => esc_html__('Education Requirement. Like: Bachelor Degree', "review-schema"),
                    ),
                    array(
                        'name'  => 'experienceRequirements',
                        'type'  => 'number',
                        'label' => esc_html__('Experience Requirements', "review-schema"),
                        'desc'  => esc_html__('Value will be number of month, Like: 2 years = 24', "review-schema")
                    ),
                    array(
                        'name'  => 'incentiveCompensation',
                        'type'  => 'text',
                        'label' => esc_html__('Incentive Compensation', "review-schema"),
                    ),
                    array(
                        'name'  => 'industry',
                        'type'  => 'text',
                        'label' => esc_html__('Industry', "review-schema"),
                    ),
                    array(
                        'name'  => 'occupationalCategory',
                        'type'  => 'text',
                        'label' => esc_html__('Occupational Category', "review-schema"),
                    ),
                    array(
                        'name'  => 'qualifications',
                        'type'  => 'textarea',
                        'label' => esc_html__('Qualifications', "review-schema"),
                    ),
                    array(
                        'name'  => 'responsibilities',
                        'type'  => 'textarea',
                        'label' => esc_html__('Responsibilities', "review-schema"),
                    ),
                    array(
                        'name'  => 'skills',
                        'type'  => 'textarea',
                        'label' => esc_html__('Skills', "review-schema"),
                    ),
                ]
            ),
            //local_business
            array(
                "type"    => "group",
                "name"    => $this->prefix . "local_business_schema", 
                "id"      => "rtrs-local_business_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Local business schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),   
                    array(
                        'name'     => 'category',
                        'label'    => esc_html__('Category', "review-schema"),
                        'type'     => 'schema_type',
                        'required' => true,
                        'options'  => Functions::getSiteTypes(),
                        'empty'    => esc_html__('Select One', "review-schema"),
                        'desc'     => esc_html__("Use the most appropriate schema category for local business", "review-schema")
                    ),
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                    ),
                    array(
                        'name'     => 'image',
                        'type'     => 'gallery',
                        'label'    => esc_html__('Images', "review-schema"),
                    ),
                    array(
                        'name'     => 'logo',
                        'type'     => 'image',
                        'label'    => esc_html__('Business Logo', "review-schema"),
                        'desc'    => esc_html__('This is only for Organization category and The image must be 112x112px, at minimum.', "review-schema"),
                    ),
                    array(
                        'name'        => 'priceRange',
                        'type'        => 'text',
                        'label'       => esc_html__('Price Range', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("The price range of the business, for example $$$.", "review-schema")
                    ),
                    array(
                        "type"    => "group",
                        "duplicate" => false,
                        "name"    => "address",   
                        "label"   => esc_html__("Address", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'  => 'streetAddress',
                                'type'  => 'text',
                                'label' => esc_html__('Street address', "review-schema"),
                            ),
                            array(
                                'name'  => 'addressLocality',
                                'type'  => 'text',
                                'label' => esc_html__('Address locality', "review-schema"),
                                'desc'  => esc_html__('City (i.e Melbourne)', "review-schema")
                            ),
                            array(
                                'name'  => 'addressRegion',
                                'type'  => 'text',
                                'label' => esc_html__('Address region', "review-schema"),
                                'desc'  => esc_html__('State (i.e. Victoria)', "review-schema")
                            ),
                            array(
                                'name'  => 'postalCode',
                                'type'  => 'text',
                                'label' => esc_html__('Postal code', "review-schema"),
                            ), 
                            array(
                                'name'     => 'addressCountry',
                                'label'    => esc_html__('Country', "review-schema"),
                                'type'     => 'select2',
                                'options'  => Functions::getCountryList(),
                                'empty'    => esc_html__('Select One', "review-schema"), 
                            ),
                        ]
                    ), 
                    array(
                        "type"    => "group",
                        "duplicate" => false,
                        "name"    => "geo",   
                        "label"   => esc_html__("GEO", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'  => 'latitude',
                                'type'  => 'text',
                                'label' => esc_html__('Latitude', "review-schema"),
                            ), 
                            array(
                                'name'  => 'longitude',
                                'type'  => 'text',
                                'label' => esc_html__('Longitude', "review-schema"),
                            ), 
                        ]
                    ), 
                    array(
                        "type"    => "group", 
                        "name"    => "opening_hours",   
                        "label"   => esc_html__("Opening Hours By Day", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'    => 'day',
                                'type'    => 'select',
                                'label'   => esc_html__('Day', "review-schema"),
                                'empty'   => esc_html__( 'Select one', 'review-schema' ),
                                'options' => array(
                                    'Monday'      => esc_html__('Monday', "review-schema"), 
                                    'Tuesday'     => esc_html__('Tuesday', "review-schema"), 
                                    'Wednesday'   => esc_html__('Wednesday', "review-schema"), 
                                    'Thursday'    => esc_html__('Thursday', "review-schema"), 
                                    'Friday'      => esc_html__('Friday', "review-schema"), 
                                    'Saturday'    => esc_html__('Saturday', "review-schema"), 
                                    'Sunday'      => esc_html__('Sunday', "review-schema"), 
                                ),
                                'desc'    => esc_html__("Don't add same day multiple time", "review-schema")
                            ),
                            array(
                                'name'  => 'opens',
                                'type'  => 'text',
                                'label' => esc_html__('Opens Time', "review-schema"),
                                'desc' => esc_html__('Times are specified using 24:00 time. For example, 3PM is specified as 15:00.', "review-schema"),
                            ),  
                            array(
                                'name'  => 'closes',
                                'type'  => 'text',
                                'label' => esc_html__('Closes Time', "review-schema"),
                            ),  
                        ]
                    ),
                    array(
                        'name'        => 'telephone',
                        'type'        => 'text',
                        'label'       => esc_html__('Telephone', "review-schema"),
                        'recommended' => true,
                    ),
                    array(
                        'name'        => 'url',
                        'type'        => 'url',
                        'label'       => esc_html__('Web URL', "review-schema"),
                        'recommended' => true,
                    ),
                    array(
                        'name'     => 'servesCuisine',
                        'type'     => 'text',
                        'label'    => esc_html__('Serves Cuisine', "review-schema"),
                        'desc'    => esc_html__('This is only for Restaurant category', "review-schema"),
                    ),
                    array(
                        'name'     => 'menu',
                        'type'     => 'url',
                        'label'    => esc_html__('Menu URL', "review-schema"),
                        'desc'    => esc_html__('This is only for Restaurant category', "review-schema"),
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ),
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Review Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),                    
                    array(
                        'name'     => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder'     => 'https://facebook.com/example&#10;https://twitter.com/example',
                        'required' => true,
                        'desc'     => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'  => 'review_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'  => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"),
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"), 
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    )
                ]
            ),
            //software_app
            array(
                "type"    => "group",
                "name"    => $this->prefix . "software_app_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-software_app_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Software app schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),   
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                    ),
                    array(
                        'name'  => 'price',
                        'type'  => 'number',
                        'label' => esc_html__('Price', "review-schema"),
                        'desc'  => esc_html__("The lowest available price, including service charges and fees, of this type of ticket.", "review-schema")
                    ),
                    array(
                        'name'  => 'priceCurrency',
                        'type'  => 'text',
                        'label' => esc_html__('Price currency', "review-schema"),
                        'desc'  => esc_html__("The 3-letter currency code.", "review-schema")
                    ),
                    array(
                        'name'     => 'applicationCategory',
                        'type'     => 'select',
                        'label'    => esc_html__('Application Category', "review-schema"),
                        'recommended' => true,
                        'options'  => self::getApplicationCategoryList(),
                        'desc'     => esc_html__("The type of app (for example, BusinessApplication or GameApplication). The value must be a supported app type.", "review-schema")
                    ),
                    array(
                        'name'  => 'operatingSystem',
                        'type'  => 'text',
                        'label' => esc_html__('Operating System', "review-schema"),
                        'desc'  => esc_html__("The operating(s) required to use the app (for example, Windows 7, OSX 10.6, Android 1.6)", "review-schema")
                    ),
                    array( 
                        'type'  => 'heading',
                        'label' => esc_html__('Aggregate Rating', "review-schema"),
                    ),
                    array(
                        'name'  => 'aggregate_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'aggregate_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"), 
                        'desc'  => esc_html__("A numerical quality rating for the item.", "review-schema")
                    ),
                    array(
                        'name'  => 'aggregate_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"), 
                        'desc'  => esc_html__("A numerical quality rating for the item.", "review-schema")
                    ),
                    array(
                        'name'  => 'aggregate_ratingCount',
                        'type'  => 'number',
                        'label' => esc_html__('Rating Count', "review-schema"), 
                        'desc'  => esc_html__("A numerical quality rating for the item.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ),
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Review Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'    => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder'     => "https://facebook.com/example&#10;https://twitter.com/example",
                        'required' => true,
                        'desc'     => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2020-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'  => 'review_ratingValue', 
                        'type'  => 'number', 
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'  => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating', 
                        'type'  => 'number', 
                        'label' => esc_html__('Best rating', "review-schema"),
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating', 
                        'type'  => 'number', 
                        'label' => esc_html__('Worst rating', "review-schema"),
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    )
                ]
            ),  
            //book
            array(
                "type"    => "group",
                "name"    => $this->prefix . "book_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-book_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Book schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__("Name", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'datePublished',
                        'type'     => 'text',
                        'label'    => esc_html__('Published date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'required' => true,
                        'desc'     => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder'     => "https://facebook.com/example&#10;https://twitter.com/example",
                        'required' => true,
                        'desc'     => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => "bookFormat",
                        'type'     => "select",
                        'label'    => esc_html__("Book Format", "review-schema"),
                        'options'  => array('EBook', 'Hardcover', 'Paperback', 'AudioBook'),
                        'required' => true
                    ),
                    array(
                        'name'     => 'isbn',
                        'type'     => 'text',
                        'label'    => esc_html__("ISBN", "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__('The ISBN of the tome. Use the ISBN of the print book instead if there is no ISBN for that edition, such as for a Kindle edition.', "review-schema")
                    ),
                    array(
                        'name'     => 'workExample',
                        'type'     => 'text',
                        'label'    => esc_html__("Work Example", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'url',
                        'type'     => 'url',
                        'label'    => esc_html__("URL", "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__('URL to the page on your site about the book. The page may list all available editions.', "review-schema")
                    ),
                    array(
                        'name'     => 'sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Same As", "review-schema"),
                        'attr'     => 'placeholder="https://example.com/example&#10;https://example.com/example"',
                        'required' => true,
                        'desc'     => esc_html__('A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry', "review-schema")
                    ),
                    array(
                        'name'  => 'publisher',
                        'type'  => 'text',
                        'label' => esc_html__("Publisher", "review-schema"),
                    ),
                    array(
                        'name'  => 'numberOfPages',
                        'type'  => 'number',
                        'label' => esc_html__("Number of Pages", "review-schema"), 
                    ),
                    array(
                        'name'  => 'copyrightHolder',
                        'type'  => 'text',
                        'label' => esc_html__("Copyright Holder", "review-schema"),
                        'desc'  => esc_html__('Holt, Rinehart and Winston', "review-schema")
                    ),
                    array(
                        'name'  => 'copyrightYear', 
                        'type'  => 'number',
                        'label' => esc_html__("Copyright Year", "review-schema"),
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'text',
                        'label' => esc_html__("Description", "review-schema"),
                    ),
                    array(
                        'name'  => 'genre',
                        'type'  => 'text',
                        'label' => esc_html__("Genre", "review-schema"),
                        'desc'  => esc_html__('Educational Materials', "review-schema")
                    ),
                    array(
                        'name'  => 'inLanguage',
                        'type'  => 'text',
                        'label' => esc_html__("Language", "review-schema"),
                        'desc'  => esc_html__('en-US', "review-schema")
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ), 
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Review status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder'     => "https://facebook.com/example&#10;https://twitter.com/example",
                        'required' => true,
                        'desc'     => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'  => 'review_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'  => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"),
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"),
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    )
                ]
            ),
            //course
            array(
                "type"    => "group",
                "name"    => $this->prefix . "course_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-course_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Course schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                    ),
                    array(
                        'name'  => 'provider',
                        'type'  => 'text',
                        'label' => esc_html__('Provider', "review-schema"),
                    ),
                    array(
                        'name'  => 'courseMode',
                        'type'  => 'textarea',
                        'label' => esc_html__('Course Mode', "review-schema"),
                        'placeholder'  => esc_html__( 'One item per line like bellow', 'review-schema' ),
                        'desc'  => wp_kses( __("MOOC<br>online", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'  => 'startDate',
                        'type'  => 'text',
                        'label' => esc_html__('Start Date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('2020-10-16', "review-schema")
                    ),
                    array(
                        'name'  => 'endDate',
                        'type'  => 'text',
                        'label' => esc_html__('End Date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('2020-10-16', "review-schema")
                    ),
                    array(
                        'name'     => 'locationName',
                        'type'     => 'text',
                        'label'    => esc_html__('Location name', "review-schema"),
                        'required' => true,
                    ),
                    array(
                        'name'     => 'locationAddress',
                        'type'     => 'text',
                        'label'    => esc_html__('Location address', "review-schema"),
                        'required' => true,
                    ),
                    array(
                        'name'  => 'image',
                        'type'  => 'image',
                        'label' => esc_html__('Course image', "review-schema"),
                    ),
                    array(
                        'name'  => 'price',
                        'type'  => 'number',
                        'label' => esc_html__('Price', "review-schema"),
                    ),
                    array(
                        'name'  => 'priceCurrency',
                        'type'  => 'text',
                        'label' => esc_html__('Price Currency', "review-schema"),
                        'desc'  => esc_html__("USD", "review-schema")
                    ),
                    array(
                        'name'    => 'availability',
                        'type'    => 'select',
                        'label'   => esc_html__('Availability', "review-schema"),
                        'empty'   => esc_html__( 'Select one', 'review-schema' ),
                        'options' => array(
                            'http://schema.org/InStock'             => esc_html__('InStock', "review-schema"),
                            'http://schema.org/InStoreOnly'         => esc_html__('InStoreOnly', "review-schema"),
                            'http://schema.org/OutOfStock'          => esc_html__('OutOfStock', "review-schema"),
                            'http://schema.org/SoldOut'             => esc_html__('SoldOut', "review-schema"),
                            'http://schema.org/OnlineOnly'          => esc_html__('OnlineOnly', "review-schema"),
                            'http://schema.org/LimitedAvailability' => esc_html__('LimitedAvailability', "review-schema"),
                            'http://schema.org/Discontinued'        => esc_html__('Discontinued', "review-schema"),
                            'http://schema.org/PreOrder'            => esc_html__('PreOrder', "review-schema"),
                        ),
                        'desc'    => esc_html__("Select a availability type", "review-schema")
                    ),
                    array(
                        'name'  => 'url',
                        'type'  => 'url',
                        'label' => esc_html__('Course Url', "review-schema"),
                    ),
                    array(
                        'name'  => 'validFrom',
                        'type'  => 'text',
                        'label' => esc_html__('Valid From', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('The date when the item becomes valid. Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'    => 'performerType',
                        'type'    => 'select',
                        'label'   => 'Performer Type',
                        'options' => array('Organization', 'Person')
                    ),
                    array(
                        'name'  => 'performerName',
                        'type'  => 'text',
                        'label' => esc_html__('Performer Name', "review-schema"),
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ),
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder' => "https://facebook.com/example&#10;https://twitter.com/example",
                        'required' => true,
                        'desc'     => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'  => 'review_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'  => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"), 
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"), 
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    )
                ]
            ),  
            //product
            array(
                "type"    => "group",
                "name"    => $this->prefix . "product_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-product_schema", 
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Product schema", 'review-schema'),  
                "fields" => [
                    /* array(
                        'name'  => 'product',
                        'type'  => 'button',
                        'label' => esc_html__('Regenerate schema', "review-schema"),
                        'value' => esc_html__('Regenerate', "review-schema"),
                        'doc'  => esc_html__("This will auto fill schema data again.", "review-schema")
                    ), */
                    array(
                        "type"      => "tab",
                        "name"      => "status",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'image',
                        'type'  => 'image',
                        'label' => esc_html__('Image', "review-schema"),
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                        'desc'  => esc_html__("Product description.", "review-schema")
                    ),
                    array(
                        'name'  => 'identifier_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Product Identifier', "review-schema"),
                        // 'desc'  => esc_html__("Add Product unique Identifier.", "review-schema")
                    ),
                    array(
                        'name'        => 'sku',
                        'type'        => 'text',
                        'label'       => esc_html__('SKU', "review-schema"),
                        'recommended' => true
                    ),
                    array(
                        'name'     => 'brand',
                        'type'     => 'text',
                        'label'    => esc_html__('Brand', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The brand of the product (Used globally).", "review-schema")
                    ),
                    array(
                        'name'     => 'identifier_type',
                        'type'     => 'select',
                        'label'    => esc_html__('Identifier Type', "review-schema"),
                        'required' => true,
                        'options'  => array(
                            'mpn'    => esc_html__('MPN', 'review-schema'),
                            'isbn'   => esc_html__('ISBN', 'review-schema'),
                            'gtin8'  => esc_html__('GTIN-8 (UPC, JAN)', 'review-schema'),
                            'gtin12' => esc_html__('GTIN-12 (UPC)', 'review-schema'),
                            'gtin13' => esc_html__('GTIN-13 (EAN,JAN)', 'review-schema')
                        ),
                        'desc' => wp_kses( __("<strong>MPN</strong><br>
                                &#8594; MPN(Manufacturer Part Number) Used globally, Alphanumeric digits (various lengths)<br>
                                <strong>GTIN</strong><br>
                                &#8594; UPC(Universal Product Code) Used in primarily North America. 12 numeric digits. eg. 892685001003.<br>
                                &#8594; EAN(European Article Number) Used primarily outside of North America. Typically 13 numeric digits (can occasionally be either eight or 14 numeric digits). eg. 4011200296908<br>
                                &#8594; ISBN(International Standard Book Number) Used globally, ISBN-13 (recommended), 13 numeric digits 978-0747595823<br>
                                &#8594; JAN(Japanese Article Number) Used only in Japan, 8 or 13 numeric digits.", "review-schema"), [ 'br' => [], 'strong' => [] ]
                            )
                    ),
                    array(
                        'name'     => 'identifier',
                        'type'     => 'text',
                        'label'    => esc_html__('Identifier Value', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("Enter product unique identifier", "review-schema")
                    ),
                    array(
                        'name'  => 'rating_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Product Review & Rating', "review-schema"),
                    ),
                    array(
                        'name'        => 'reviewRatingValue',
                        'type'        => 'number',
                        'label'       => esc_html__('Review rating avg value', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'        => 'reviewBestRating',
                        'type'        => 'number',
                        'label'       => esc_html__('Review Best rating', "review-schema"),
                        'recommended' => true,
                    ),
                    array(
                        'name'        => 'reviewWorstRating',
                        'type'        => 'number',
                        'label'       => esc_html__('Review Worst rating', "review-schema"),
                        'recommended' => true,
                    ),
                    array(
                        'name'  => 'reviewAuthor',
                        'type'  => 'text',
                        'label' => esc_html__('Review author', "review-schema"),
                    ),
                    array(
                        'name'        => 'ratingValue',
                        'type'        => 'number',
                        'label'       => esc_html__('Aggregate Rating value', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'reviewCount',
                        'type'  => 'number',
                        'label' => esc_html__('Aggregate Total review count', "review-schema"),
                        'desc'  => wp_kses( __("Review Count. <span class='required'>This is required if (Rating value) is given</span>", "review-schema"), [ 'span' => [] ] )
                    ),
                    array(
                        'name'  => 'pricing_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Product Pricing', "review-schema"),
                    ),
                    array(
                        'name'  => 'priceCurrency',
                        'type'  => 'text',
                        'label' => esc_html__('Price currency', "review-schema"),
                        'desc'  => esc_html__("The 3-letter currency code.", "review-schema")
                    ),
                    array(
                        'name'  => 'price',
                        'type'  => 'number',
                        'label' => esc_html__('Price', "review-schema"),
                        'desc'  => esc_html__("The lowest available price, including service charges and fees, of this type of ticket.", "review-schema")
                    ),
                    array(
                        'name'        => 'priceValidUntil',
                        'type'        => 'text',
                        'label'       => esc_html__('PriceValidUntil', "review-schema"),
                        'recommended' => true,
                        'class'       => 'rtrs-date',
                        'desc'        => esc_html__("The date (in ISO 8601 date format) after which the price will no longer be available. Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'    => 'availability',
                        'type'    => 'select',
                        'label'   => esc_html__('Availability', "review-schema"),
                        'empty'   => esc_html__( 'Select one', 'review-schema' ),
                        'options' => array(
                            'http://schema.org/InStock'             => esc_html__('InStock', "review-schema"),
                            'http://schema.org/InStoreOnly'         => esc_html__('InStoreOnly', "review-schema"),
                            'http://schema.org/OutOfStock'          => esc_html__('OutOfStock', "review-schema"),
                            'http://schema.org/SoldOut'             => esc_html__('SoldOut', "review-schema"),
                            'http://schema.org/OnlineOnly'          => esc_html__('OnlineOnly', "review-schema"),
                            'http://schema.org/LimitedAvailability' => esc_html__('LimitedAvailability', "review-schema"),
                            'http://schema.org/Discontinued'        => esc_html__('Discontinued', "review-schema"),
                            'http://schema.org/PreOrder'            => esc_html__('PreOrder', "review-schema"),
                        ),
                        'desc'    => esc_html__("Select a availability type", "review-schema")
                    ),
                    array(
                        'name'    => 'itemCondition',
                        'type'    => 'select',
                        'label'   => esc_html__( 'Product condition', 'review-schema' ),
                        'empty'   => esc_html__( 'Select one', 'review-schema' ),
                        'options' => array(
                            'http://schema.org/NewCondition'         => esc_html__('NewCondition', "review-schema"),
                            'http://schema.org/UsedCondition'        => esc_html__('UsedCondition', "review-schema"),
                            'http://schema.org/DamagedCondition'     => esc_html__('DamagedCondition', "review-schema"),
                            'http://schema.org/RefurbishedCondition' => esc_html__('RefurbishedCondition', "review-schema"),
                        ),
                        'desc'    => esc_html__("Select a condition", "review-schema")
                    ),
                    array(
                        'name'  => 'url',
                        'type'  => 'url',
                        'label' => esc_html__('Product URL', "review-schema"),
                        'desc'  => esc_html__("A URL to the product web page (that includes the Offer). (Don't use offerURL for markup that appears on the product page itself.)", "review-schema")
                    )
                ]
            ),  
            //recipe
            array(
                "type"    => "group",
                "name"    => $this->prefix . "recipe_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-recipe_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Recipe schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        'name'  => 'name',
                        'type'  => 'text',
                        'label' => esc_html__('Name', "review-schema"),
                    ),
                    array(
                        'name'  => 'author',
                        'type'  => 'text',
                        'label' => esc_html__('Author', "review-schema"),
                    ),
                    array(
                        'name'  => 'datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Published Date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'image',
                        'type'     => 'image',
                        'label'    => esc_html__('Image', "review-schema"),
                        'required' => true,
                    ),
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"),
                    ),
                    array(
                        'name'        => 'keywords',
                        'type'        => 'text',
                        'label'       => esc_html__('Recipe keywords', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("Pizza, Nice, Testy", "review-schema")
                    ),
                    array(
                        'name'        => 'recipeCategory',
                        'type'        => 'text',
                        'label'       => esc_html__('Recipe Category', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("example, appetizer, entree, etc.", "review-schema")
                    ),
                    array(
                        'name'        => 'recipeCuisine',
                        'type'        => 'text',
                        'label'       => esc_html__('Recipe Cuisine', "review-schema"),
                        'recommended' => true,
                        'desc'        => esc_html__("example, French or Ethiopian", "review-schema")
                    ), 
                    array(
                        'name'  => 'prepTime',
                        'type'  => 'text',
                        'label' => esc_html__('Prepare Time', "review-schema"),
                        'desc'  => esc_html__('PT15M', "review-schema")
                    ),
                    array(
                        'name'  => 'cookTime',
                        'type'  => 'text',
                        'label' => esc_html__('Cook Time', "review-schema"),
                        'desc'  => esc_html__('PT1H', "review-schema")
                    ),  
                    array(
                        "type"    => "group",
                        "name"    => "ingredient",   
                        "label"   => esc_html__("Recipe Ingredient", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"),
                                'required' => true
                            ), 
                        ]
                    ), 
                    array(
                        "type"    => "group",
                        "name"    => "instructions",   
                        "label"   => esc_html__("Recipe Instructions", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"), 
                            ), 
                            array(
                                'name'     => 'text',
                                'type'     => 'textarea',
                                'label'    => esc_html__('Text', "review-schema"), 
                            ), 
                            array(
                                'name'  => 'image',
                                'type'  => 'image',
                                'label' => esc_html__('Image', "review-schema"),
                            ),
                            array(
                                'name'     => 'url',
                                'type'     => 'url',
                                'label'    => esc_html__('URL', "review-schema"), 
                            ),  
                        ]
                    ),
                    array(
                        'name'  => 'calories',
                        'type'  => 'text',
                        'label' => esc_html__('Nutrition: calories', "review-schema"),
                        'desc'  => esc_html__('240 calories', "review-schema")
                    ),
                    array(
                        'name'  => 'fatContent',
                        'type'  => 'text',
                        'label' => esc_html__('Nutrition: Fat Content', "review-schema"),
                        'desc'  => esc_html__('9 grams fat', "review-schema")
                    ),
                    array(
                        'name'  => 'userInteractionCount', 
                        'type'  => 'number',
                        'label' => esc_html__('User Interaction Count', "review-schema"),
                    ),
                    array(
                        'name'  => 'ratingValue', 
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'reviewCount', 
                        'type'  => 'number',
                        'label' => esc_html__('Review Count', "review-schema"),
                    ),
                    array(
                        'name'  => 'bestRating', 
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"),
                    ),
                    array(
                        'name'  => 'worstRating', 
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"),
                    ),
                    array(
                        'name'  => 'recipeYield',
                        'type'  => 'text',
                        'label' => esc_html__('Recipe Yield', "review-schema"),
                    ),
                    array(
                        'name'  => 'suitableForDiet',
                        'type'  => 'text',
                        'label' => esc_html__('Suitable ForDiet', "review-schema"),
                        'desc'  => esc_html__('http://schema.org/LowFatDiet', "review-schema")
                    ),
                    array(
                        "type"    => "group",
                        "duplicate" => false,
                        "name"    => "video",   
                        "label"   => esc_html__("Video Info", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'  => 'description',
                                'type'  => 'textarea',
                                'label' => esc_html__('Description', "review-schema"),
                                'required' => true 
                            ), 
                            array(
                                'name'  => 'thumbnailUrl',
                                'type'  => 'image',
                                'label' => esc_html__('Image', "review-schema"),
                                'required' => true
                            ),
                            array(
                                'name'     => 'contentUrl',
                                'type'     => 'url',
                                'label'    => esc_html__('Content URL', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'     => 'embedUrl',
                                'type'     => 'url',
                                'label'    => esc_html__('Embed URL', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'     => 'uploadDate',
                                'type'     => 'text',
                                'label'    => esc_html__('Upload date', "review-schema"),
                                'class' => 'rtrs-date',
                                'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'     => 'duration',
                                'type'     => 'text',
                                'label'    => esc_html__('Duration', "review-schema"),
                                'desc'  => esc_html__('Runtime of the movie in ISO 8601 format (for example, "PT2H22M" (142 minutes)).', "review-schema"),
                                'required' => true 
                            ), 
                        ]
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ),
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Review Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder' => 'https://facebook.com/example&#10;https://twitter.com/example',
                        'required' => true,
                        'desc'     => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'  => 'review_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"),
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"), 
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    )
                ]
            ), 
            //faq
            array(
                "type"    => "group",
                "duplicate" => false,
                "name"    => $this->prefix . "faq_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-faq_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Faq schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        "type"    => "group",
                        "name"    => "faqs",  
                        "id"      => "rtrs-faq_schema",
                        "label"   => esc_html__("Faqs", 'review-schema'),  
                        "fields" => [ 
                            array(
                                "name"   => "ques",
                                "type"   => "text",
                                "label"  => esc_html__("Question", 'review-schema'),  
                            ), 
                            array(
                                "name"   => "ans",
                                "type"   => "textarea",
                                "label"  => esc_html__("Answer", 'review-schema'),  
                            ), 
                        ]
                    ),
                ]
            ), 
            //how_to
            array(
                "type"    => "group",
                "name"    => $this->prefix . "how_to_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-how_to_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("How to schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true
                    ), 
                    array(
                        'name'  => 'description',
                        'type'  => 'textarea',
                        'label' => esc_html__('Description', "review-schema"), 
                    ), 
                    array(
                        'name'  => 'image',
                        'type'  => 'image',
                        'label' => esc_html__('Image', "review-schema"),
                    ), 
                    array(
                        'name'  => 'priceCurrency',
                        'type'  => 'text',
                        'label' => esc_html__('Price currency', "review-schema"),
                        'desc'  => esc_html__("The 3-letter currency code.", "review-schema")
                    ),
                    array(
                        'name'  => 'price',
                        'type'  => 'number',
                        'label' => esc_html__('Price', "review-schema"),
                    ),
                    array(
                        "type"    => "group",
                        "name"    => "supply",   
                        "label"   => esc_html__("Supply", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"),
                                'required' => true
                            ), 
                        ]
                    ),
                    array(
                        "type"    => "group",
                        "name"    => "tool",   
                        "label"   => esc_html__("Tool", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"),
                                'required' => true
                            ), 
                        ]
                    ),
                    array(
                        "type"    => "group",
                        "name"    => "step",   
                        "label"   => esc_html__("Step", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"), 
                            ), 
                            array(
                                'name'     => 'text',
                                'type'     => 'textarea',
                                'label'    => esc_html__('Text', "review-schema"), 
                            ), 
                            array(
                                'name'  => 'image',
                                'type'  => 'image',
                                'label' => esc_html__('Image', "review-schema"),
                            ),
                            array(
                                'name'     => 'url',
                                'type'     => 'url',
                                'label'    => esc_html__('URL', "review-schema"), 
                            ), 
                            array(
                                'name'     => 'clipId',
                                'type'     => 'text',
                                'label'    => esc_html__('Video Clip ID', "review-schema"), 
                                'desc'     => esc_html__('Video Clip ID, this need to match with Video Info Clip ID', "review-schema"), 
                            ),
                            array(
                                "type"    => "group",
                                "name"    => "direction",   
                                "label"   => esc_html__("Direction", 'review-schema'),  
                                "fields" => [ 
                                    array(
                                        'name'     => 'text',
                                        'type'     => 'textarea',
                                        'label'    => esc_html__('Description', "review-schema"),
                                        'required' => true
                                    ),  
                                ]
                            ),
                        ]
                    ),
                    array(
                        'name'     => 'totalTime',
                        'type'     => 'text',
                        'label'    => esc_html__('Total time', "review-schema"), 
                    ), 
                    array(
                        "type"    => "group",
                        "duplicate" => false,
                        "name"    => "video",   
                        "label"   => esc_html__("Video Info", 'review-schema'),  
                        "fields" => [ 
                            array(
                                'name'     => 'name',
                                'type'     => 'text',
                                'label'    => esc_html__('Name', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'  => 'description',
                                'type'  => 'textarea',
                                'label' => esc_html__('Description', "review-schema"),
                                'required' => true 
                            ), 
                            array(
                                'name'  => 'thumbnailUrl',
                                'type'  => 'image',
                                'label' => esc_html__('Image', "review-schema"),
                                'required' => true
                            ),
                            array(
                                'name'     => 'contentUrl',
                                'type'     => 'url',
                                'label'    => esc_html__('Content URL', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'     => 'embedUrl',
                                'type'     => 'url',
                                'label'    => esc_html__('Embed URL', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'     => 'uploadDate',
                                'type'     => 'text',
                                'label'    => esc_html__('Upload date', "review-schema"),
                                'class' => 'rtrs-date',
                                'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                'name'     => 'duration',
                                'type'     => 'text',
                                'label'    => esc_html__('Duration', "review-schema"),
                                'desc'  => esc_html__('Runtime of the movie in ISO 8601 format (for example, "PT2H22M" (142 minutes)).', "review-schema"),
                                'required' => true 
                            ),
                            array(
                                "type"    => "group",
                                "name"    => "clip",   
                                "label"   => esc_html__("Clip", 'review-schema'),  
                                "fields" => [ 
                                    array(
                                        'name'     => 'name',
                                        'type'     => 'text',
                                        'label'    => esc_html__('Name', "review-schema"), 
                                    ), 
                                    array(
                                        'name'     => 'id',
                                        'type'     => 'text',
                                        'label'    => esc_html__('Clip ID', "review-schema"), 
                                        'desc'    => esc_html__('Like: Clip1, Clip2 etc', "review-schema"), 
                                    ), 
                                    array(
                                        'name'     => 'startOffset',
                                        'type'     => 'number',
                                        'label'    => esc_html__('Start Offset', "review-schema"), 
                                    ), 
                                    array(
                                        'name'     => 'endOffset',
                                        'type'     => 'number',
                                        'label'    => esc_html__('End Offset', "review-schema"), 
                                    ), 
                                    array(
                                        'name'     => 'url',
                                        'type'     => 'url',
                                        'label'    => esc_html__('URL', "review-schema"), 
                                    ), 
                                ]
                            ),
                        ]
                    ),
                ]
            ), 
            //question_answer
            array(
                "type"    => "group",
                "name"    => $this->prefix . "question_answer_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-question_answer_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Q&A schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        "name"   => "name",
                        "type"   => "text",
                        "label"  => esc_html__("Question title", 'review-schema'),  
                    ), 
                    array(
                        "name"   => "text",
                        "type"   => "textarea",
                        "label"  => esc_html__("Question description", 'review-schema'),  
                    ),
                    array(
                        "name"   => "answerCount",
                        "type"   => "number",
                        "label"  => esc_html__("Question total answer", 'review-schema'),  
                    ), 
                    array(
                        "name"   => "upvoteCount",
                        "type"   => "number",
                        "label"  => esc_html__("Question total votes", 'review-schema'),  
                    ), 
                    array(
                        'name'  => 'dateCreated',
                        'type'  => 'text',
                        'label' => esc_html__('Question created Date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ), 
                    array(
                        "name"   => "author",
                        "type"   => "text",
                        "label"  => esc_html__("Question author", 'review-schema'),  
                    ), 
                    array(
                        "type"    => "group",
                        "name"    => "answers",  
                        "id"      => "rtrs-question_answer_schema",
                        "label"   => esc_html__("Answers", 'review-schema'),  
                        "fields" => [  
                            array(
                                "name"   => "text",
                                "type"   => "textarea",
                                "label"  => esc_html__("Answer", 'review-schema'),  
                            ), 
                            array(
                                'name'  => 'dateCreated',
                                'type'  => 'text',
                                'label' => esc_html__('Created Date', "review-schema"),
                                'class' => 'rtrs-date',
                                'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                            ),
                            array(
                                "name"   => "upvoteCount",
                                "type"   => "number",
                                "label"  => esc_html__("Total votes", 'review-schema'),  
                            ), 
                            array(
                                "name"   => "url",
                                "type"   => "url",
                                "label"  => esc_html__("URL", 'review-schema'),  
                            ), 
                            array(
                                "name"   => "author",
                                "type"   => "text",
                                "label"  => esc_html__("Author", 'review-schema'),  
                            ), 
                            array(
                                "name"      => "answerType",
                                "type"      => "tab",
                                "label"     => esc_html__("Answer type", 'review-schema'), 
                                "desc"      => esc_html__("Only one accepted answer will be in this list.", 'review-schema'),
                                "default"   => 'normal',
                                "options"   => [
                                    'normal' => esc_html__("General", 'review-schema'), 
                                    'accepted' => esc_html__("Accepted", 'review-schema'), 
                                ]
                            ),  
                        ]
                    ),
                ]
            ),  
            //breadcrumb
            array(
                "type"    => "group",
                "name"    => $this->prefix . "breadcrumb_schema",  
                "id"      => "rtrs-breadcrumb_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Breadcrumb schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        "type"    => "group",
                        "name"    => "items",  
                        "id"      => "rtrs-breadcrumb_schema",
                        "label"   => esc_html__("Items", 'review-schema'),  
                        "fields" => [ 
                            array(
                                "name"   => "name",
                                "type"   => "text",
                                "label"  => esc_html__("Name", 'review-schema'),  
                            ), 
                            array(
                                "name"   => "item",
                                "type"   => "url",
                                "label"  => esc_html__("URL", 'review-schema'),   
                            ), 
                        ]
                    ),
                ]
            ), 
            //itemlist
            array(
                "type"    => "group",
                "name"    => $this->prefix . "itemlist_schema",  
                "id"      => "rtrs-itemlist_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Itemlist schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),  
                    array(
                        "type"    => "group",
                        "name"    => "items",  
                        "id"      => "rtrs-itemlist_schema",
                        "label"   => esc_html__("Items", 'review-schema'),  
                        "fields" => [  
                            array(
                                "name"   => "url",
                                "type"   => "url",
                                "label"  => esc_html__("URL", 'review-schema'),   
                            ), 
                        ]
                    ),
                ]
            ),  
            //movie
            array(
                "type"    => "group",
                "name"    => $this->prefix . "movie_schema", 
                // "is_pro"  => true,
                "id"      => "rtrs-movie_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Movie schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'description',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Description', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'duration',
                        'type'  => 'text',
                        'label' => esc_html__('Duration', "review-schema"),
                        'desc'  => esc_html__('Runtime of the movie in ISO 8601 format (for example, "PT2H22M" (142 minutes)).', "review-schema")
                    ),
                    array(
                        'name'  => 'dateCreated',
                        'type'  => 'text',
                        'label' => esc_html__('Created Date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('Like this: 2019-12-25 14:20:00', "review-schema")
                    ),
                    array(
                        'name'     => 'image',
                        'type'     => 'image',
                        'label'    => esc_html__('Image', "review-schema"),
                        'required' => true,
                    ),
                    array(
                        'name'  => 'director',
                        'type'  => 'text',
                        'label' => esc_html__('Director', "review-schema"),
                    ),
                    array(
                        'name'  => 'author',
                        'type'  => 'textarea',
                        'label' => esc_html__('Author', "review-schema"),
                        'placeholder' => esc_html__('One item per line like bellow', "review-schema"),
                        'desc'  => wp_kses( __("Ted Elliott<br>Terry Rossio", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'  => 'actor',
                        'type'  => 'textarea',
                        'label' => esc_html__('Actor', "review-schema"),
                        'placeholder' => esc_html__('One item per line like bellow', "review-schema"),
                        'desc'  => wp_kses( __("Johnny Depp<br>Penelope Cruz<br>Ian McShane", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'  => 'review_section',
                        'type'  => 'heading',
                        'label' => esc_html__('Review', "review-schema"),
                        'desc'  => esc_html__("To add review schema for this type, complete fields below and enable, others live blank.", "review-schema")
                    ), 
                    array(
                        "name"      => "review_active",
                        "type"      => "tab",
                        "label"     => esc_html__("Review status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ),
                    array(
                        'name'     => 'review_author',
                        'type'     => 'text',
                        'label'    => esc_html__("Author", "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'     => 'review_author_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Author Same As profile link", "review-schema"),
                        'placeholder'     => 'https://facebook.com/example&#10;https://twitter.com/example',
                        'required' => true,
                        'desc' => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_body',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Review body', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The actual body of the review.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_datePublished',
                        'type'  => 'text',
                        'label' => esc_html__('Date of Published', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                    array(
                        'name'     => 'review_publisher',
                        'type'     => 'text',
                        'label'    => esc_html__('Publisher', "review-schema"),
                        'desc'     => esc_html__('Publisher name or Organization name', "review-schema"),
                        'required' => true
                    ),
                    array(
                        'name'  => 'review_publisherImage',
                        'type'  => 'image',
                        'label' => esc_html__('Publisher Logo', "review-schema"),
                        'desc' => wp_kses( __("Logos should have a wide aspect ratio, not a square icon.<br>Logos should be no wider than 600px, and no taller than 60px.<br>Always retain the original aspect ratio of the logo when resizing. Ideally, logos are exactly 60px tall with width <= 600px. If maintaining a height of 60px would cause the width to exceed 600px, downscale the logo to exactly 600px wide and reduce the height accordingly below 60px to maintain the original aspect ratio.", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'     => 'review_sameAs',
                        'type'     => 'textarea',
                        'label'    => esc_html__("Review same as link", "review-schema"),
                        'placeholder'     => 'https://example.com/example&#10;https://example.com/example',
                        'required' => true,
                        'desc' => wp_kses( __("A reference page that unambiguously indicates the item\'s identity; for example, the URL of the item\'s Wikipedia page, Freebase page, or official website.<br> Enter new line for every entry", "review-schema"), [ 'br' => [] ] )
                    ),
                    array(
                        'name'  => 'review_ratingValue',
                        'type'  => 'number',
                        'label' => esc_html__('Rating avg value', "review-schema"), 
                        'desc'        => esc_html__("Rating value. (1, 2.5, 3, 5 etc)", "review-schema")
                    ),
                    array(
                        'name'  => 'review_bestRating',
                        'type'  => 'number',
                        'label' => esc_html__('Best rating', "review-schema"),
                        'desc'  => esc_html__("The highest value allowed in this rating.", "review-schema")
                    ),
                    array(
                        'name'  => 'review_worstRating',
                        'type'  => 'number',
                        'label' => esc_html__('Worst rating', "review-schema"),
                        'desc'  => esc_html__("The lowest value allowed in this rating. * Required if the rating is not on a 5-point scale. If worstRating is omitted, 1 is assumed.", "review-schema")
                    )
                ]
            ),
            //audio
            array(
                "type"    => "group",
                "name"    => $this->prefix . "audio_schema", 
                "id"      => "rtrs-audio_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Video schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The title of the audio", "review-schema")
                    ),
                    array(
                        'name'     => 'description',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Description', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The short description of the audio", "review-schema")
                    ), 
                    array(
                        'name'  => 'duration',
                        'type'  => 'text',
                        'required' => true,
                        'label' => esc_html__('Duration', "review-schema"),
                        'desc'  => esc_html__("The duration of the audio in ISO 8601 format.(PT1M33S)", "review-schema")
                    ),
                    array(
                        'name'        => 'contentUrl',
                        'type'        => 'url',
                        'required' => true,
                        'label'       => esc_html__('Content URL', "review-schema"),
                        'placeholder' => esc_html__('URL', "review-schema"),
                        'desc'        => esc_html__("A URL pointing to the actual audio media file. This file should be in .mp3, .wav, .mpc or other audio file format.", "review-schema")
                    ),
                    array(
                        'name'  => 'encodingFormat',
                        'type'  => 'text',
                        'label' => esc_html__('Encoding Format', "review-schema"),
                        'desc'  => esc_html__("The encoding format of audio like: 'audio/mpeg'", "review-schema")                    ),
                ]
            ),
            //video
            array(
                "type"    => "group",
                "name"    => $this->prefix . "video_schema", 
                "id"      => "rtrs-video_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Video schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The title of the video", "review-schema")
                    ),
                    array(
                        'name'     => 'description',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Description', "review-schema"),
                        'required' => true,
                        'desc'     => esc_html__("The description of the video", "review-schema")
                    ),
                    array(
                        'name'        => 'thumbnailUrl',
                        'type'        => 'url',
                        'label'       => esc_html__( 'Thumbnail URL', 'review-schema' ),
                        'placeholder' => esc_html__( 'URL', 'review-schema' ),
                        'required'    => true,
                        'desc'        => esc_html__("A URL pointing to the video thumbnail image file. Images must be at least 160x90 pixels and at most 1920x1080 pixels.", "review-schema")
                    ),
                    array(
                        'name'  => 'uploadDate',
                        'type'  => 'text',
                        'label' => esc_html__('Updated date', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__('2020-02-05T08:00:00+08:00', "review-schema")
                    ),
                    array(
                        'name'  => 'duration',
                        'type'  => 'text',
                        'label' => esc_html__('Duration', "review-schema"),
                        'desc'  => esc_html__("The duration of the video in ISO 8601 format.(PT1M33S)", "review-schema")
                    ),
                    array(
                        'name'        => 'contentUrl',
                        'type'        => 'url',
                        'label'       => esc_html__('Content URL', "review-schema"),
                        'placeholder' => esc_html__('URL', "review-schema"),
                        'desc'        => esc_html__("A URL pointing to the actual video media file. This file should be in .mpg, .mpeg, .mp4, .m4v, .mov, .wmv, .asf, .avi, .ra, .ram, .rm, .flv, or other video file format.", "review-schema")
                    ),
                    array(
                        'name'        => 'embedUrl',
                        'type'        => 'url',
                        'label'       => esc_html__('Embed URL', "review-schema"),
                        'placeholder' => esc_html__('URL', "review-schema"),
                        'desc'        => esc_html__("A URL pointing to a player for the specific video. Usually this is the information in the src element of an &lt;embed> tag.Example: Dailymotion: http://www.dailymotion.com/swf/x1o2g.", "review-schema")
                    ),
                    array(
                        'name'  => 'interactionCount',
                        'type'  => 'text',
                        'label' => esc_html__('Interaction count', "review-schema"),
                        'desc'  => esc_html__("The number of times the video has been viewed.", "review-schema")
                    ),
                    array(
                        'name'  => 'expires',
                        'type'  => 'text',
                        'label' => esc_html__('Expires', "review-schema"),
                        'class' => 'rtrs-date',
                        'desc'  => esc_html__("Like this: 2019-12-25 14:20:00", "review-schema")
                    ),
                ]
            ),
            //image_license
            array(
                "type"    => "group",
                "name"    => $this->prefix . "image_license_schema", 
                "is_pro"  => true,
                "id"      => "rtrs-image_license_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Image License schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    array(
                        'name'        => 'contentUrl',
                        'type'        => 'url',
                        'label'       => esc_html__( 'Content URL', 'review-schema' ),
                        'placeholder' => esc_html__( 'URL', 'review-schema' ),
                        'required'    => true,
                        'desc'        => esc_html__("Image URL", "review-schema")
                    ),
                    array(
                        'name'        => 'license',
                        'type'        => 'url',
                        'label'       => esc_html__( 'License URL', 'review-schema' ),
                        'placeholder' => esc_html__( 'URL', 'review-schema' ),
                        'required'    => true, 
                    ),
                    array(
                        'name'        => 'acquireLicensePage',
                        'type'        => 'url',
                        'label'       => esc_html__( 'Acquire license URL', 'review-schema' ),
                        'placeholder' => esc_html__( 'URL', 'review-schema' ),
                        'required'    => true, 
                    ),
                ]
            ), 
            //special_announcement
            array(
                "type"    => "group",
                "name"    => $this->prefix . "special_announcement_schema", 
                "id"      => "rtrs-special_announcement_schema",
                "holderClass" => "rtrs-hidden rtrs-schema-field",
                "label"   => esc_html__("Special announcement schema", 'review-schema'),  
                "fields" => [
                    array(
                        "name"      => "status",
                        "type"      => "tab",
                        "label"     => esc_html__("Status", 'review-schema'), 
                        "default"   => 'show',
                        "options"   => [
                            'show' => esc_html__("Show", 'review-schema'), 
                            'hide' => esc_html__("Hide", 'review-schema'), 
                        ]
                    ), 
                    [
                        'name'     => 'name',
                        'type'     => 'text',
                        'label'    => esc_html__('Name', "review-schema"),
                        'desc'     => esc_html__('SpecialAnnouncement.name: Name of the announcement. This text should be present on the underlying page.', "review-schema"),
                        'required' => true
                    ],
                    [
                        'name'     => 'url',
                        'type'     => 'url',
                        'label'    => esc_html__('Page URL', "review-schema"),
                        'desc'     => esc_html__('SpecialAnnouncement.url: URL of the page containing the announcements. If present, this must match the URL of the page containing the information.', "review-schema"),
                        'required' => true
                    ],
                    [
                        'name'     => 'datePublished',
                        'type'     => 'text',
                        'label'    => esc_html__('Published date', "review-schema"),
                        'class'    => 'rtrs-date',
                        'desc'     => esc_html__('Like this: 2020-12-25 14:20:00', "review-schema"),
                        'required' => true
                    ],
                    [
                        'name'        => 'expires',
                        'type'        => 'text',
                        'label'       => esc_html__('Expires date', "review-schema"),
                        'class'       => 'rtrs-date',
                        'desc'        => esc_html__('Like this: 2020-12-25 14:20:00', "review-schema"),
                        'recommended' => true,
                    ],
                    [
                        'name'     => 'text',
                        'type'     => 'textarea',
                        'label'    => esc_html__('Text', "review-schema"),
                        'desc'     => esc_html__('SpecialAnnouncement.text: Text of the announcements.', "review-schema"),
                        'required' => true
                    ],
                    [
                        'name'      => 'locations',
                        'type'      => 'group',
                        'label'     => esc_html__('Announcement Locations', "review-schema"), 
                        'fields'    => [ 
                            [
                                'name'     => 'type',
                                'type'     => 'select',
                                'label'    => esc_html__('Type', "review-schema"),
                                'options'  => self::announcementLocationTypes(),
                                'required' => true
                            ],
                            [
                                'name'        => 'name',
                                'type'        => 'text',
                                'label'       => esc_html__('Name', "review-schema"),
                                'desc'        => esc_html__("SpecialAnnouncement.announcementLocation.name: ", "review-schema"),
                                'recommended' => true,
                            ],
                            [
                                'name'        => 'url',
                                'type'        => 'url',
                                'label'       => esc_html__('URL', "review-schema"),
                                'recommended' => true,
                                'desc'        => esc_html__("SpecialAnnouncement.announcementLocation.url: URL", "review-schema")
                            ],
                            [
                                'name'        => 'address_street',
                                'type'        => 'text',
                                'label'       => esc_html__('Address: Street', "review-schema"),
                                'desc'        => esc_html__('SpecialAnnouncement.announcementLocation.address.streetAddress: The street address. For example, 1600 Amphitheatre Pkwy.', "review-schema"),
                                'recommended' => true,
                            ],
                            [
                                'name'        => 'address_locality',
                                'type'        => 'text',
                                'label'       => esc_html__('Address: Locality', "review-schema"),
                                'desc'        => esc_html__('SpecialAnnouncement.announcementLocation.address.addressLocality: The locality in which the street address is, and which is in the region. For example, Mountain View.', "review-schema"),
                                'recommended' => true,
                            ],
                            [
                                'name'        => 'address_post_code',
                                'type'        => 'text',
                                'label'       => esc_html__('Address: Post Code', "review-schema"),
                                'desc'        => esc_html__('SpecialAnnouncement.announcementLocation.address.postalCode: The postal code. For example, 94043.', "review-schema"),
                                'recommended' => true,
                            ],
                            [
                                'name'        => 'address_region',
                                'type'        => 'text',
                                'label'       => esc_html__('Address: Region', "review-schema"),
                                'desc'        => esc_html__('SpecialAnnouncement.announcementLocation.address.addressRegion: The region in which the locality is, and which is in the country. For example, California.', "review-schema"),
                                'recommended' => true,
                            ],
                            [
                                'name'        => 'address_country',
                                'type'        => 'text',
                                'label'       => esc_html__('Address: Country', "review-schema"),
                                'desc'        => esc_html__('SpecialAnnouncement.announcementLocation.address.addressCountry: The country. For example, USA. You can also provide the two-letter ISO 3166-1 alpha-2 country code.', "review-schema"),
                                'recommended' => true,
                            ],
                            [
                                'name'  => 'id',
                                'type'  => 'text',
                                'label' => esc_html__('ID', "review-schema"),
                                'desc'  => esc_html__('SpecialAnnouncement.announcementLocation.@id: An optional unique identifier so that you can reference pre-existing structured data for this location.', "review-schema"),
                            ],
                            array(
                                'name'  => 'image',
                                'type'  => 'image',
                                'label' => esc_html__('Image', "review-schema"),
                            ),
                            array(
                                'name'        => 'priceRange',
                                'type'        => 'text',
                                'label'       => 'Price Range (Recommended)',
                                'recommended' => true,
                                'desc'        => esc_html__("The price range of the business, for example $$$.", "review-schema")
                            ),
                            array(
                                'name'        => 'telephone',
                                'type'        => 'text',
                                'label'       => esc_html__( 'Telephone (Recommended)', 'review-schema' ),
                                'recommended' => true
                            )
                        ]
                    ],
                ]
            ), 
        );
        return apply_filters('rtrs_section_schema_fields', $settings_fields);
    }   

    static function getApplicationCategoryList() {

        $list = array(
            "GameApplication",
            "SocialNetworkingApplication",
            "TravelApplication",
            "ShoppingApplication",
            "SportsApplication",
            "LifestyleApplication",
            "BusinessApplication",
            "DesignApplication",
            "DeveloperApplication",
            "DriverApplication",
            "EducationalApplication",
            "HealthApplication",
            "FinanceApplication",
            "SecurityApplication",
            "BrowserApplication",
            "CommunicationApplication",
            "DesktopEnhancementApplication",
            "EntertainmentApplication",
            "MultimediaApplication",
            "HomeApplication",
            "UtilitiesApplication",
            "ReferenceApplication",
        );

        $new_list = [];
        foreach ($list as $value) {
            $new_list[$value] = $value;
        }

        return apply_filters('rtseo_application_category_list', $new_list);
    }

    static function announcementLocationTypes() {
        $location_types = [
            'Airport',
            'Aquarium',
            'Beach',
            'Bridge',
            'BuddhistTemple',
            'BusStation',
            'BusStop',
            'Campground',
            'CatholicChurch',
            'Cemetery',
            'Church',
            'CivicStructure',
            'CityHall',
            'CollegeOrUniversity',
            'Courthouse',
            'CovidTestingFacility',
            'Crematorium',
            'DefenceEstablishment',
            'EducationalOrganization',
            'ElementarySchool',
            'Embassy',
            'EventVenue',
            'FireStation',
            'GovernmentBuilding',
            'HighSchool',
            'HinduTemple',
            'Hospital',
            'LegislativeBuilding',
            'MiddleSchool',
            'Mosque',
            'MovieTheater',
            'Museum',
            'MusicVenue',
            'Park',
            'ParkingFacility',
            'PerformingArtsTheater',
            'PlaceOfWorship',
            'Playground',
            'PoliceStation',
            'Preschool',
            'RVPark',
            'School',
            'StadiumOrArena',
            'SubwayStation',
            'Synagogue',
            'TaxiStand',
            'TrainStation',
            'Zoo',
        ];

        $new_location_types = [];
        foreach ($location_types as $value) {
            $new_location_types[$value] = $value;
        }

        return apply_filters('rtseo_announcement_location_types', $new_location_types);
    }
}