<?php 

namespace Rtrs\Models; 
use Rtrs\Helpers\Functions;
use Rtrs\Models\Review;

class Schema {
    
    function __construct() {
        add_action( 'wp_footer', array( $this, 'footer' ), 999 );
    }

    /**
     * Get social links array
     *
     * @since 1.0
     * @return array
     */
    function get_social() {       
        $social_links = rtrs()->get_options('rtrs_schema_social_profiles_settings', ['social_profiles', []]); 
        
        // Remove empty fields
        $social = array();
        foreach( $social_links as $profile ) {
            if ( $profile['url'] ) $social[] = $profile['url'];
        }
        
        return $social;
    }

    /**
     * Get Get corporate contacts types array
     *
     * @since 1.0
     * @return array
     */
    function corporate_contacts_types() {
        
        $corporate_contacts	= array();
        
        $corporate_contacts = rtrs()->get_options('rtrs_schema_corporate_contacts_settings'); 

        $contact_type = isset( $corporate_contacts['type'] ) ? esc_attr( $corporate_contacts['type'] ) : '';
         
        if ( $contact_type )  {
            
            // Remove dashes and replace it with a space 
            $contact_type	= str_replace("_", " ", $contact_type);
        
            $corporate_contacts = array(
                '@type'			=> 'ContactPoint',	// default required value
                'contactType'	=> $contact_type,
                'telephone'		=> ( $corporate_contacts['telephone'] ) ? $corporate_contacts['telephone'] : '',
                'url'			=> ( $corporate_contacts['url'] ) ? esc_url( $corporate_contacts['url'] ) : '',
                'email'			=> ( $corporate_contacts['email'] ) ? esc_html( $corporate_contacts['email'] ) : '',
                'contactOption'	=> ( $corporate_contacts['contactOption'] ) ? esc_html( $corporate_contacts['contactOption'] ) : '',
                'areaServed'	=> ( $corporate_contacts['areaServed'] ) ? $corporate_contacts['areaServed'] : '',
                'availableLanguage'	=> ( $corporate_contacts['availableLanguage'] ) ? $corporate_contacts['availableLanguage'] : '',
            );
        }
        
        return $corporate_contacts;
    }

    /**
     * Google sitelink searchbox
     * @return mixed
     */
    function footer() {
        $settings = get_option( 'rtrs_schema_settings' );

        // site schema
        $enable_site_schema = false;
        
        $site_schema = !empty($settings['site_schema']) ? $settings['site_schema'] : 'home_page';
        if ( $site_schema !== 'off' ) {
            if ( $site_schema == 'home_page' ) {
                if (is_home() || is_front_page()) {
                    $enable_site_schema = true;
                }
            } elseif ($site_schema == 'all') {
                $enable_site_schema = true;
            }
        }
 
        if ( $enable_site_schema ) { 
            
            $helper = new Functions();

            //sitelink searchbox
            $settings_sitelink = get_option( 'rtrs_schema_sitelink_settings' );
            if ( isset( $settings_sitelink['sitelink_searchbox'] ) && $settings_sitelink['sitelink_searchbox'] == 'yes' ) {
                $html = null; 
                $metaData = array(); 
                $metaData["@context"] = "http://schema.org/";
                $metaData["@type"]    = "WebSite";  
                $metaData["url"]      = trailingslashit( get_home_url() );
                $metaData["potentialAction"] = array(
                    "@type"       => "SearchAction",
                    "target"      => trailingslashit( get_home_url() ) . "?s={query}",
                    "query-input" => "required name=query"
                ); 
                echo $this->getJsonEncode(apply_filters('rtseo_sitelink_searchbox', $metaData)); 
            }
        
            $metaData = $settings;
            $category = !empty($metaData['category']) ? $helper->sanitizeOutPut($metaData['category']) : "LocalBusiness";
            $local_business = array(
                "@context" => "https://schema.org",
                "@type"    => $category
            );

            if ( !empty($metaData['url']) ) {
                $local_business["@id"] = $helper->sanitizeOutPut($metaData['url'], 'url');
            }
                
            if ( !empty($metaData['name']) ) {
                $local_business["name"] = $helper->sanitizeOutPut($metaData['name']);
            }

            if ( !empty($metaData['alternateName']) ) {
                $local_business["alternateName"] = $helper->sanitizeOutPut($metaData['alternateName']);
            } 

            if ( !empty($metaData['description']) ) {
                $local_business["description"] = $helper->sanitizeOutPut($metaData['description'], 'textarea');
            } 

            if ( !empty($metaData['image']) ) {
                $img = $helper->imageInfo(absint($metaData['image']));
                $local_business["image"][] = $helper->sanitizeOutPut($img['url'], 'url');
            }

            if ( !empty($metaData['logo']) ) {
                $img = $helper->imageInfo(absint($metaData['logo']));
                $local_business["logo"] = $helper->sanitizeOutPut($img['url'], 'url');
            }

            if ( !empty($metaData['priceRange']) ) {
                $local_business["priceRange"] = $helper->sanitizeOutPut($metaData['priceRange']);
            } 

            if ( $this->get_social() ) {
                $local_business["sameAs"] = $this->get_social();
            }
 
            if ( !empty($metaData['sameAs']) ) {
                $aType = explode("\r\n", $metaData['sameAs']);

                if ( $local_business["sameAs"] ) {
                    $aType = array_merge($aType, $local_business["sameAs"]);
                }
                
                if ( !empty($aType) && is_array($aType) ) {
                    if ( count($aType) == 1 ) {
                        $local_business["sameAs"] = $aType[0];
                    } else if (count($aType) > 1) {
                        $local_business["sameAs"] = $aType;
                    }
                }
            }

            if ( $this->corporate_contacts_types() ) {
                $local_business["contactPoint"] = $this->corporate_contacts_types();
            }

            if ( $category == 'Restaurant' ) {
                if ( !empty($metaData['servesCuisine']) ) {
                    $local_business["servesCuisine"] = $helper->sanitizeOutPut($metaData['servesCuisine']); 
                }

                if ( !empty($metaData['menu']) ) {
                    $local_business["menu"] = $helper->sanitizeOutPut($metaData['menu'], 'url');
                    if ( isset( $metaData['acceptsReservations'] ) && $metaData['acceptsReservations'] == 'yes' ) { 
                        $local_business["acceptsReservations"] = "True";
                    }
                }
            }

            if ( !empty($metaData['addresses']) ) {
                $addresses = [];
                $address_i = 1;
                foreach ($metaData['addresses'] as $address) {
                    if (!empty($address['addressLocality']) || !empty($address['addressRegion'])
                || !empty($address['postalCode']) || !empty($address['streetAddress'])) {
                        if( !function_exists('rtrsp') && $address_i > 1 ) break;
                        
                        $addresses[] = array(
                            "@type"           => "PostalAddress",
                            "addressLocality" => $helper->sanitizeOutPut($address['addressLocality']),
                            "addressRegion"   => $helper->sanitizeOutPut($address['addressRegion']),
                            "postalCode"      => $helper->sanitizeOutPut($address['postalCode']),
                            "streetAddress"   => $helper->sanitizeOutPut($address['streetAddress']),
                            "addressCountry"  => $helper->sanitizeOutPut($address['addressCountry'])
                        );
                        $address_i++;
                    }
                }

                if ( $addresses ) {
                    $local_business["address"] = $addresses;
                }
            }

            $metaDataSubOrg = get_option('rtrs_schema_sub_organization_settings');
            if ( function_exists('rtrsp') && !empty($metaDataSubOrg['sub_organization']) ) {
                $sub_organization = [];
                foreach ( $metaDataSubOrg['sub_organization'] as $sub_org ) {
                    if ( !empty($sub_org['name']) || !empty($sub_org['url']) ) {
                        $sub_organization[] = array(
                            "@type"           => "Organization",
                            "name" => $helper->sanitizeOutPut($sub_org['name']),
                            "url"   => $helper->sanitizeOutPut($sub_org['url']) 
                        );
                    }
                }

                if ( $sub_organization ) {
                    $local_business["subOrganization"] = $sub_organization;
                }
            }

            if ( !empty($metaData['latitude']) || !empty($metaData['longitude']) ) { 
                $local_business["geo"] = array(
                    "@type"       => "GeoCircle",
                    "geoMidpoint" => array(
                        "@type"     => "GeoCoordinates",
                        "latitude"  => $helper->sanitizeOutPut($metaData['latitude']),
                        "longitude" => $helper->sanitizeOutPut($metaData['longitude']),
                    ),
                    "geoRadius"   => !empty($metaData['radius']) ? $helper->sanitizeOutPut($metaData['radius']) : 50,
                );
            }

            if (!empty($metaData['telephone'])) {
                $local_business["telephone"] = $helper->sanitizeOutPut($metaData['telephone']);
            }

            if (!empty($metaData['url'])) {
                $local_business["url"] = $helper->sanitizeOutPut($metaData['url'], 'url');
            }

            if ( ! empty( $metaData['openingHours'] ) ) {
                $opening_day = explode( "\r\n", $metaData['openingHours'] );
                if ( ! empty( $opening_day ) && is_array( $opening_day ) ) { 
                    $local_business_opening_hours = array();
                    foreach ( $opening_day as $value ) {
                        $opening_day_time = explode( " ", $value );
                        $opening_day = isset( $opening_day_time[0] ) ? $opening_day_time[0] : '';

                        $opening_time = isset( $opening_day_time[1] ) ? $opening_day_time[1] : '';
                        $opening_time = explode( "-", $opening_time );

                        $open_time = isset( $opening_time[0] ) ? $opening_time[0] : '';
                        $closes_time = isset( $opening_time[1] ) ? $opening_time[1] : '';

                        $opening_hours_single_schema = array(
                            "@type"          => "OpeningHoursSpecification",
                            "dayOfWeek"      => $opening_day ? $helper->sanitizeOutPut($opening_day) : '', 
                            "opens"          => $open_time ? $helper->sanitizeOutPut($open_time) : '', 
                            "closes"         => $closes_time ? $helper->sanitizeOutPut($closes_time) : '', 
                        );
                        array_push($local_business_opening_hours, $opening_hours_single_schema); 
                    } 
                    $local_business['openingHoursSpecification'] = $local_business_opening_hours;
                }
            } 

            echo $this->getJsonEncode(apply_filters('rtseo_site_schema', $local_business, $metaData)); 

            // Generate SiteNavigationElement Schema
            $sitelink_settings = get_option( 'rtrs_schema_sitelink_settings' );

            if ( !empty($sitelink_settings['site_nav']) ) {
                $items = wp_get_nav_menu_items(absint($sitelink_settings['site_nav']));
                if (!empty($items)) {
                    $navData = array();
                    $navData["@context"] = "https://schema.org/";
                    $itemData = array();
                    foreach ($items as $item) {
                        $itemData[] = array(
                            "@context" => "https://schema.org",
                            "@type"    => "SiteNavigationElement",
                            "@id"      => "#table-of-contents",
                            "name"     => esc_html($item->title),
                            "url"      => esc_url($item->url)
                        );
                    }
                    $navData["@graph"] = $itemData; 
                    echo $this->getJsonEncode(apply_filters('rtseo_sitelink_nav_menu', $navData)); 
                }
            }         
        } 
    }

    /**
     * rich snippet generator
     * @return object
     */
    public function rich_snippet() {
    
        if ( !is_singular() ) return;      
        
        global $post;
        $post_type = $post->post_type;
        $p_meta = Functions::getMetaByPostType( $post_type );
        $rich_snippet = ( isset( $p_meta['rich_snippet'] ) && $p_meta['rich_snippet'][0] == '1' ); 
        $rich_snippet_cat = isset( $p_meta['rich_snippet_cat'] ) ? $p_meta['rich_snippet_cat'][0] : null; 

        $prefix = 'rtrs_'; 
        $output = null;
        $post_id = get_the_ID();
        $custom_snippet = get_post_meta( $post_id, '_rtrs_custom_rich_snippet', true );

        if ( $custom_snippet ) {
            $schemaCat = get_post_meta( $post_id, '_rtrs_rich_snippet_cat', false );
            foreach ( $schemaCat as $singleCat ) {
                $metaData = get_post_meta( $post_id, $prefix.$singleCat.'_schema', true );
                foreach ($metaData as $meta) {
                    if ( $meta['status'] == 'show' ) {
                        $output .= $this->schemaOutput($singleCat, $meta); 
                    }  
                }
            }
        } else { //auto generate
            if ( $rich_snippet ) {
                $output = $this->autoSchemaOutput($rich_snippet_cat, $post_id);
            }
        }
        $generated_type = ( $custom_snippet ) ? 'manually' : 'auto';
        return "\n" . '<!-- This Google structured data (Rich Snippet) '.$generated_type.' generated by RadiusTheme Review Schema plugin version '.RTRS_VERSION.' -->' . "\n" . $output . "\n";
    }

    // private static $generated_text = false;

    function getJsonEncode($data = array()) {
        $html = null;
        if ( !empty($data) && is_array($data) ) {
            //show only one time
            /* if ( self::$generated_text === false ) {
                $html .= '<!-- This Google structured data (Rich Snippet) generated by RadiusTheme Review Schema plugin version '.RTRS_VERSION.' -->';
                self::$generated_text = true;
            }  */
            
            $html .= '<script type="application/ld+json">' . json_encode($data,
                    JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES) . '</script>';
        }

        return $html;
    }

    /**
     * schema output
     * @return object
     */
    public function schemaOutput( $schemaCat, $metaData = null) {
        
        $html = null;
        if ( $schemaCat ) {  
            $helper = new Functions();
            switch ( $schemaCat ) {
                case "article":
                    $article = array(
                        "@context" => "https://schema.org",
                        "@type"    => "Article"
                    );
                    if (!empty($metaData['headline'])) {
                        $article["headline"] = $helper->sanitizeOutPut($metaData['headline']);
                    }
                    if (!empty($metaData['mainEntityOfPage'])) {
                        $article["mainEntityOfPage"] = array(
                            "@type" => "WebPage",
                            "@id"   => $helper->sanitizeOutPut($metaData["mainEntityOfPage"])
                        );
                    }
                    if (!empty($metaData['author'])) {
                        $article["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['author'])
                        );

                        if ( !empty($metaData['author_url']) ) {
                            $article["author"]['url'] =  $helper->sanitizeOutPut($metaData['author_url'], 'url');
                        } 
                    }
                    if (!empty($metaData['publisher'])) {
                        if (!empty($metaData['publisherImage'])) {
                            $img = $helper->imageInfo(absint($metaData['publisherImage']));
                            $plA = array(
                                "@type"  => "ImageObject",
                                "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                                "height" => $img['height'],
                                "width"  => $img['width']
                            );
                        } else {
                            $plA = array();
                        }
                        $article["publisher"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['publisher']),
                            "logo"  => $plA
                        );
                    }
                    if (!empty($metaData['alternativeHeadline'])) {
                        $article["alternativeHeadline"] = $helper->sanitizeOutPut($metaData['alternativeHeadline']);
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $article["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }
                    if (!empty($metaData['datePublished'])) {
                        $article["datePublished"] = $helper->sanitizeOutPut($metaData['datePublished']);
                    }
                    if (!empty($metaData['dateModified'])) {
                        $article["dateModified"] = $helper->sanitizeOutPut($metaData['dateModified']);
                    }
                    if (!empty($metaData['description'])) {
                        $article["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['articleBody'])) {
                        $article["articleBody"] = $helper->sanitizeOutPut($metaData['articleBody'],
                            'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_article', $article, $metaData));
                    break;

                case "news_article":
                    $newsArticle = array(
                        "@context" => "https://schema.org",
                        "@type"    => "NewsArticle"
                    );
                    if (!empty($metaData['headline'])) {
                        $newsArticle["headline"] = $helper->sanitizeOutPut($metaData['headline']);
                    }
                    if (!empty($metaData['mainEntityOfPage'])) {
                        $newsArticle["mainEntityOfPage"] = array(
                            "@type" => "WebPage",
                            "@id"   => $helper->sanitizeOutPut($metaData["mainEntityOfPage"])
                        );
                    }
                    if (!empty($metaData['author'])) {
                        $newsArticle["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['author'])
                        );

                        if ( !empty($metaData['author_url']) ) {
                            $newsArticle["author"]['url'] =  $helper->sanitizeOutPut($metaData['author_url'], 'url');
                        } 
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $newsArticle["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }
                    if (!empty($metaData['datePublished'])) {
                        $newsArticle["datePublished"] = $helper->sanitizeOutPut($metaData['datePublished']);
                    }
                    if (!empty($metaData['dateModified'])) {
                        $newsArticle["dateModified"] = $helper->sanitizeOutPut($metaData['dateModified']);
                    }
                    if (!empty($metaData['publisher'])) {
                        if (!empty($metaData['publisherImage'])) {
                            $img = $helper->imageInfo(absint($metaData['publisherImage']));
                            $plA = array(
                                "@type"  => "ImageObject",
                                "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                                "height" => $img['height'],
                                "width"  => $img['width']
                            );
                        } else {
                            $plA = array();
                        }
                        $newsArticle["publisher"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['publisher']),
                            "logo"  => $plA
                        );
                    }
                    if (!empty($metaData['description'])) {
                        $newsArticle["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['articleBody'])) {
                        $newsArticle["articleBody"] = $helper->sanitizeOutPut($metaData['articleBody'],
                            'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_news_article', $newsArticle, $metaData));
                    break;

                case "blog_posting":
                    $blogPosting = array(
                        "@context" => "https://schema.org",
                        "@type"    => "BlogPosting"
                    );
                    if (!empty($metaData['headline'])) {
                        $blogPosting["headline"] = $helper->sanitizeOutPut($metaData['headline']);
                    }
                    if (!empty($metaData['mainEntityOfPage'])) {
                        $blogPosting["mainEntityOfPage"] = array(
                            "@type" => "WebPage",
                            "@id"   => $helper->sanitizeOutPut($metaData["mainEntityOfPage"])
                        );
                    }
                    if (!empty($metaData['author'])) {
                        $blogPosting["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['author'])
                        );

                        if ( !empty($metaData['author_url']) ) {
                            $blogPosting["author"]['url'] =  $helper->sanitizeOutPut($metaData['author_url'], 'url');
                        }
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $blogPosting["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }
                    if (!empty($metaData['datePublished'])) {
                        $blogPosting["datePublished"] = $helper->sanitizeOutPut($metaData['datePublished']);
                    }
                    if (!empty($metaData['dateModified'])) {
                        $blogPosting["dateModified"] = $helper->sanitizeOutPut($metaData['dateModified']);
                    }
                    if (!empty($metaData['publisher'])) {
                        if (!empty($metaData['publisherImage'])) {
                            $img = $helper->imageInfo(absint($metaData['publisherImage']));
                            $plA = array(
                                "@type"  => "ImageObject",
                                "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                                "height" => $img['height'],
                                "width"  => $img['width']
                            );
                        } else {
                            $plA = array();
                        }
                        $blogPosting["publisher"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['publisher']),
                            "logo"  => $plA
                        );
                    }
                    if (!empty($metaData['description'])) {
                        $blogPosting["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['articleBody'])) {
                        $blogPosting["articleBody"] = $helper->sanitizeOutPut($metaData['articleBody'],
                            'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_blog_posting', $blogPosting, $metaData));
                    break;
                
                case 'course':
                    $course = array();
                    $course["@context"] = "http://schema.org";
                    $course["@type"] = "Course";
                    $course["hasCourseInstance"] = array(
                        "@type" => "CourseInstance"
                    );
                    if (!empty($metaData['name'])) {
                        $course["name"] = $course["hasCourseInstance"]["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $course["description"] = $course["hasCourseInstance"]["description"] = $helper->sanitizeOutPut($metaData['description']);
                    }
                    if (!empty($metaData['provider'])) {
                        $course["provider"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['provider'])
                        );
                    }

                    if (!empty($metaData['courseMode'])) {
                        $course["hasCourseInstance"]["courseMode"] = explode("\r\n",
                            $helper->sanitizeOutPut($metaData['courseMode'], 'textarea'));

                    }
                    if (!empty($metaData['endDate'])) {
                        $course["hasCourseInstance"]["endDate"] = $helper->sanitizeOutPut($metaData['endDate']);
                    }
                    if (!empty($metaData['startDate'])) {
                        $course["hasCourseInstance"]["startDate"] = $helper->sanitizeOutPut($metaData['startDate']);
                    }
                    if (!empty($metaData['locationName']) && !empty($metaData['locationAddress'])) {
                        $course["hasCourseInstance"]["location"] = array(
                            "@type"   => "Place",
                            "name"    => $helper->sanitizeOutPut($metaData['locationName']),
                            "address" => $helper->sanitizeOutPut($metaData['locationAddress'])
                        );
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $course["hasCourseInstance"]["image"] = $helper->sanitizeOutPut($img['url'],
                            'url');
                    }
                    if (!empty($metaData['price']) && !empty($metaData['priceCurrency'])) {
                        $course["hasCourseInstance"]["offers"] = array(
                            "@type"         => 'Offer',
                            "price"         => $helper->sanitizeOutPut($metaData['price']),
                            "priceCurrency" => $helper->sanitizeOutPut($metaData['priceCurrency']),
                        );
                        if (!empty($metaData['availability'])) {
                            $course["hasCourseInstance"]["offers"]["availability"] = $helper->sanitizeOutPut($metaData['availability']);
                        }
                        if (!empty($metaData['url'])) {
                            $course["hasCourseInstance"]["offers"]["url"] = $helper->sanitizeOutPut($metaData['url']);
                        }
                        if (!empty($metaData['validFrom'])) {
                            $course["hasCourseInstance"]["offers"]["validFrom"] = $helper->sanitizeOutPut($metaData['validFrom']);
                        }
                    }
                    if (!empty($metaData['performerType']) && !empty($metaData['performerName'])) {
                        $course["hasCourseInstance"]["performer"] = array(
                            "@type" => $helper->sanitizeOutPut($metaData['performerType']),
                            "name"  => $helper->sanitizeOutPut($metaData['performerName'])
                        );
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_course', $course, $metaData));
                    if ( isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $course_review = array(
                            "@context"     => "http://schema.org",
                            "@type"        => "Review",
                            "itemReviewed" => array(
                                "@type" => "Course",
                            )
                        );

                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $course_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $course_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }
                        if (isset($course["name"])) {
                            $course_review["itemReviewed"]['name'] = $course["name"];
                        }

                        if (isset($course["description"])) {
                            $course_review["itemReviewed"]["description"] = Functions::filter_content($course["description"], 200);
                        }
                        if (isset($course["provider"])) {
                            $course_review["itemReviewed"]['provider'] = $course["provider"];
                        }
                        if (!empty($metaData['review_author'])) {
                            $course_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $course_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }
                        if (isset($metaData['review_ratingValue'])) {
                            $course_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $course_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $course_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }
                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_course_review', $course_review, $metaData));
                    }
                    break;

                case 'event':
                    $event = array(
                        "@context" => "https://schema.org",
                        "@type"    => "Event"
                    );
                    if (!empty($metaData['name'])) {
                        $event["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['startDate'])) {
                        $event["startDate"] = $helper->sanitizeOutPut($metaData['startDate']);
                    }
                    if (!empty($metaData['endDate'])) {
                        $event["endDate"] = $helper->sanitizeOutPut($metaData['endDate']);
                    }
                    if (!empty($metaData['description'])) {
                        $event["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['performerName'])) {
                        $event["performer"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['performerName'])
                        );
                    }
                    if (!empty($metaData['image'])) {
                        $event["image"] = $helper->sanitizeOutPut($metaData['image'], 'url');
                    }
                    if (!empty($metaData['locationName'])) {
                        $event["location"] = array(
                            "@type"   => "Place",
                            "name"    => $helper->sanitizeOutPut($metaData['locationName']),
                            "address" => $helper->sanitizeOutPut($metaData['locationAddress'])
                        );
                    }
                    if (!empty($metaData['price'])) {
                        $event["offers"] = array(
                            "@type" => "Offer",
                            "price" => $helper->sanitizeOutPut($metaData['price'])
                        );
                        if (!empty($metaData['priceCurrency'])) {
                            $event["offers"]['priceCurrency'] = $helper->sanitizeOutPut($metaData['priceCurrency']);
                        }
                        if (!empty($metaData['url'])) {
                            $event["offers"]['url'] = $helper->sanitizeOutPut($metaData['url'], 'url');
                        }
                        if (!empty($metaData['availability'])) {
                            $event["offers"]['availability'] = $helper->sanitizeOutPut($metaData['availability']);
                        }
                        if (!empty($metaData['validFrom'])) {
                            $event["offers"]['validFrom'] = $helper->sanitizeOutPut($metaData['validFrom']);
                        }
                    }
                    if (!empty($metaData['eventStatus'])) {
                        $event['eventStatus'] = $helper->sanitizeOutPut($metaData['eventStatus']);
                    }
                    if (!empty($metaData['eventAttendanceMode'])) {
                        $event['eventAttendanceMode'] = $helper->sanitizeOutPut($metaData['eventAttendanceMode']);
                    }
                    if (!empty($metaData['url'])) {
                        $event["url"] = $helper->sanitizeOutPut($metaData['url'], 'url');
                    }
                    if (!empty($metaData['organizerName'])) {
                        $event["organizer"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['organizerName']),
                            "url"  => $helper->sanitizeOutPut($metaData['url'], 'url')
                        );
                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_event', $event, $metaData));
                    if (isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $event_review = array(
                            "@context" => "https://schema.org",
                            "@type"    => "Review"
                        );

                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $event_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $event_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }
                        unset($event['@context']);
                        $event_review["itemReviewed"] = $event;
                        if (!empty($metaData['review_author'])) {
                            $event_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $event_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }

                        if ( isset($metaData['review_ratingValue']) ) {
                            $event_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $event_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $event_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }

                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_event_review', $event_review, $metaData));
                    }
                    break;

                case 'product':
                    if ( !function_exists('rtrsp') ) break;
                    
                    $product = array(
                        "@context" => "https://schema.org",
                        "@type"    => "Product"
                    );
                    if (!empty($metaData['name'])) {
                        $product["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $product["image"] = $helper->sanitizeOutPut($img['url'], 'url');
                    }
                    if (!empty($metaData['description'])) {
                        $product["description"] = $helper->sanitizeOutPut($metaData['description']);
                    }
                    /* product identifier */
                    if (!empty($metaData['sku'])) {
                        $product["sku"] = $helper->sanitizeOutPut($metaData['sku']);
                    }
                    if (!empty($metaData['brand'])) {
                        $product["brand"] = $helper->sanitizeOutPut($metaData['brand']);
                    }
                    if (!empty($metaData['identifier_type']) && !empty($metaData['identifier'])) {
                        $product[$metaData['identifier_type']] = $helper->sanitizeOutPut($metaData['identifier']);
                    }

                    if (!empty($metaData['ratingValue'])) {
                        $product["aggregateRating"] = array(
                            "@type"       => "AggregateRating",
                            "ratingValue" => !empty($metaData['ratingValue']) ? $helper->sanitizeOutPut($metaData['ratingValue']) : null,
                            "reviewCount" => !empty($metaData['reviewCount']) ? $helper->sanitizeOutPut($metaData['reviewCount']) : null
                        );
                    }
                    
                    if (!empty($metaData['reviewRatingValue']) || !empty($metaData['reviewBestRating']) || !empty($metaData['reviewWorstRating'])) {
                        $product["review"] = array(
                            "@type"        => "Review",
                            "reviewRating" => array(
                                "@type"       => "Rating",
                                "ratingValue" => !empty($metaData['reviewRatingValue']) ? $helper->sanitizeOutPut($metaData['reviewRatingValue']) : null,
                                "bestRating"  => !empty($metaData['reviewBestRating']) ? $helper->sanitizeOutPut($metaData['reviewBestRating']) : null,
                                "worstRating" => !empty($metaData['reviewWorstRating']) ? $helper->sanitizeOutPut($metaData['reviewWorstRating']) : null
                            ),
                            "author"  => array(
                                "@type" => "Person",
                                "name"  => !empty($metaData['reviewAuthor']) ? $helper->sanitizeOutPut($metaData['reviewAuthor']) : null
                            )
                        );
                    }
                    if (!empty($metaData['price'])) {
                        $product["offers"] = array(
                            "@type"           => "Offer",
                            "price"           => $helper->sanitizeOutPut($metaData['price']),
                            "priceValidUntil" => $helper->sanitizeOutPut($metaData['priceValidUntil']),
                            "priceCurrency"   => !empty($metaData['priceCurrency']) ? $helper->sanitizeOutPut($metaData['priceCurrency']) : null,
                            "itemCondition"   => !empty($metaData['itemCondition']) ? $helper->sanitizeOutPut($metaData['itemCondition']) : null,
                            "availability"    => !empty($metaData['availability']) ? $helper->sanitizeOutPut($metaData['availability']) : null,
                            "url"             => !empty($metaData['url']) ? $helper->sanitizeOutPut($metaData['url']) : null
                        );
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_product', $product, $metaData));
                    break;

                case 'recipe':
                    $recipe = array(
                        "@context" => "http://schema.org",
                        "@type"    => "Recipe"
                    );
                    if (!empty($metaData['name'])) {
                        $recipe["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['author'])) {
                        $recipe["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['author'])
                        );
                    }
                    if (!empty($metaData['datePublished'])) {
                        $recipe["datePublished"] = $helper->sanitizeOutPut($metaData['datePublished']);
                    }
                    if (!empty($metaData['keywords'])) {
                        $recipe["keywords"] = $helper->sanitizeOutPut($metaData['keywords']);
                    }
                    if (!empty($metaData['recipeCategory'])) {
                        $recipe["recipeCategory"] = $helper->sanitizeOutPut($metaData['recipeCategory']);
                    }
                    if (!empty($metaData['recipeCuisine'])) {
                        $recipe["recipeCuisine"] = $helper->sanitizeOutPut($metaData['recipeCuisine']);
                    } 
                    if (!empty($metaData['prepTime'])) {
                        $recipe["prepTime"] = $helper->sanitizeOutPut($metaData['prepTime']);
                    }
                    if (!empty($metaData['cookTime'])) {
                        $recipe["cookTime"] = $helper->sanitizeOutPut($metaData['cookTime']);
                    }
                    if (!empty($metaData['description'])) {
                        $recipe["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $recipe["image"] = $helper->sanitizeOutPut($img['url'], 'url');
                    } 
                    
                    if ( isset($metaData['ingredient']) && is_array($metaData['ingredient']) ) {
                        $recipeIngredient = array();
                        foreach ($metaData['ingredient'] as $recipeIngredient_single) {
                            if ( !empty($recipeIngredient_single['name']) ) {
                                array_push($recipeIngredient, $helper->sanitizeOutPut($recipeIngredient_single['name'])); 
                            } 
                        }  
                        $recipe['recipeIngredient'] = $recipeIngredient;
                    } 

                    if ( isset($metaData['instructions']) && is_array($metaData['instructions']) ) {
                        $recipeinstructions = array();
                        foreach ($metaData['instructions'] as $instructions_single) {
                            $instructions_single_schema = array(
                                "@type"   => "HowToStep", 
                            );

                            if ( !empty( $instructions_single['name'] ) ) {
                                $instructions_single_schema["name"] = $helper->sanitizeOutPut($instructions_single['name']);
                            }

                            if ( !empty( $instructions_single['text'] ) ) {
                                $instructions_single_schema["text"] =  $helper->sanitizeOutPut($instructions_single['text'], 'textarea');
                            }

                            if ( !empty( $instructions_single['url'] ) ) {
                                $instructions_single_schema["url"] = $helper->sanitizeOutPut($instructions_single['url'], 'url');
                            }

                            if ( !empty( $instructions_single['image'] ) ) {
                                $img = $helper->imageInfo(absint($instructions_single['image']));
                                $instructions_single_schema["image"] = $helper->sanitizeOutPut($img['url'], 'url');
                            } 

                            array_push($recipeinstructions, $instructions_single_schema); 
                        } 
                        $recipe['recipeInstructions'] = $recipeinstructions;
                    }

                    if ( isset($metaData['video']) && is_array($metaData['video']) ) {
                        $recipevideo = array();
                        foreach ($metaData['video'] as $video_single) {
                            if ( $video_single['name'] && $video_single['contentUrl'] ) {
                                $video_single_schema = array(
                                    "@type"         => "VideoObject",
                                    "name"          => $video_single['name'] ? $helper->sanitizeOutPut($video_single['name']) : null, 
                                    "description"   => $video_single['description'] ? $helper->sanitizeOutPut($video_single['description']) : null,  
                                    "contentUrl"    => $video_single['contentUrl'] ? $helper->sanitizeOutPut($video_single['contentUrl']) : null, 
                                    "embedUrl"      => $video_single['embedUrl'] ? $helper->sanitizeOutPut($video_single['embedUrl']) : null, 
                                    "uploadDate"    => $video_single['uploadDate'] ? $helper->sanitizeOutPut($video_single['uploadDate']) : null, 
                                    "duration"      => $video_single['duration'] ? $helper->sanitizeOutPut($video_single['duration']) : null, 
                                );
                                if (!empty($video_single['thumbnailUrl'])) {
                                    $img = $helper->imageInfo(absint($video_single['thumbnailUrl']));
                                    $video_single_schema["thumbnailUrl"] = $helper->sanitizeOutPut($img['url'], 'url');
                                }
    
                                $recipevideo = $video_single_schema; 
                                 
                            } 
                        } 
                        if ( $recipevideo ) {
                            $recipe['video'] = $recipevideo;
                        }
                    } 

                    if (!empty($metaData['userInteractionCount'])) {
                        $recipe["interactionStatistic"] = array(
                            "@type"                => "InteractionCounter",
                            "interactionType"      => "http://schema.org/Comment",
                            "userInteractionCount" => $helper->sanitizeOutPut($metaData['userInteractionCount'])
                        );
                    }
                    if (!empty($metaData['ratingValue']) || !empty($metaData['reviewCount'])) {
                        $recipe["aggregateRating"] = array(
                            "@type"       => "AggregateRating",
                            "ratingValue" => $helper->sanitizeOutPut($metaData['ratingValue'], 'number'),
                            "reviewCount" => $helper->sanitizeOutPut($metaData['reviewCount'], 'number'),
                            "bestRating"  => $helper->sanitizeOutPut($metaData['bestRating'], 'number'),
                            "worstRating" => $helper->sanitizeOutPut($metaData['worstRating'], 'number')
                        );
                    }
                    if (!empty($metaData['calories']) || !empty($metaData['fatContent'])) {
                        $recipe["nutrition"] = array(
                            "@type"      => "NutritionInformation",
                            "calories"   => $helper->sanitizeOutPut($metaData['calories']),
                            "fatContent" => $helper->sanitizeOutPut($metaData['fatContent'])
                        );
                    } 
                    if (!empty($metaData['recipeYield'])) {
                        $recipe["recipeYield"] = $helper->sanitizeOutPut($metaData['recipeYield']);
                    }
                    if (!empty($metaData['suitableForDiet'])) {
                        $recipe["suitableForDiet"] = $helper->sanitizeOutPut($metaData['suitableForDiet']);
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_recipe', $recipe, $metaData));
                    if (isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $recipe_review = array(
                            "@context"     => "http://schema.org",
                            "@type"        => "Review",
                            "itemReviewed" => array(
                                "@type" => "Recipe",
                            )
                        );

                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $recipe_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        } else if (isset($recipe['datePublished'])) {
                            $recipe_review["datePublished"] = $recipe['datePublished'];
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $recipe_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }
                        unset($recipe['@context']);
                        unset($recipe['@context']);
                        $recipe_review["itemReviewed"] = $recipe;
                        if (!empty($metaData['review_author'])) {
                            $recipe_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $recipe_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }
                        if (isset($metaData['review_ratingValue'])) {
                            $recipe_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $recipe_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $recipe_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }
                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_recipe_review', $recipe_review, $metaData));
                    }
                    break;

                case 'audio':
                    $video = array(
                        "@context" => "https://schema.org",
                        "@type"    => "AudioObject"
                    );
                    if (!empty($metaData['name'])) {
                        $video["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $video["description"] = $helper->sanitizeOutPut($metaData['description'], 'textarea');
                    }  
                    if (!empty($metaData['duration'])) {
                        $video["duration"] = $helper->sanitizeOutPut($metaData['duration']);
                    }
                    if (!empty($metaData['contentUrl'])) {
                        $video["contentUrl"] = $helper->sanitizeOutPut($metaData['contentUrl'], 'url');
                    }
                    if (!empty($metaData['encodingFormat'])) {
                        $video["encodingFormat"] = $helper->sanitizeOutPut($metaData['encodingFormat']);
                    } 
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_video', $video, $metaData));
                    break;

                case 'video':
                    $video = array(
                        "@context" => "https://schema.org",
                        "@type"    => "VideoObject"
                    );
                    if (!empty($metaData['name'])) {
                        $video["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $video["description"] = $helper->sanitizeOutPut($metaData['description'], 'textarea');
                    }
                    if (!empty($metaData['thumbnailUrl'])) {
                        $video["thumbnailUrl"] = $helper->sanitizeOutPut($metaData['thumbnailUrl'], 'url');
                    }
                    if (!empty($metaData['uploadDate'])) {
                        $video["uploadDate"] = $helper->sanitizeOutPut($metaData['uploadDate']);
                    }
                    if (!empty($metaData['duration'])) {
                        $video["duration"] = $helper->sanitizeOutPut($metaData['duration']);
                    }
                    if (!empty($metaData['contentUrl'])) {
                        $video["contentUrl"] = $helper->sanitizeOutPut($metaData['contentUrl'], 'url');
                    }
                    if (!empty($metaData['embedUrl'])) {
                        $video["embedUrl"] = $helper->sanitizeOutPut($metaData['embedUrl'], 'url');
                    }
                    if (!empty($metaData['interactionCount'])) {
                        $video["interactionCount"] = $helper->sanitizeOutPut($metaData['interactionCount']);
                    }
                    if (!empty($metaData['expires'])) {
                        $video["expires"] = $helper->sanitizeOutPut($metaData['expires']);
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_video', $video, $metaData));
                    break;

                case 'image_license':
                    $image_license = array(
                        "@context" => "https://schema.org",
                        "@type"    => "ImageObject"
                    );
                     
                    if (!empty($metaData['contentUrl'])) {
                        $image_license["contentUrl"] = $helper->sanitizeOutPut($metaData['contentUrl'], 'url');
                    }
                    if (!empty($metaData['license'])) {
                        $image_license["license"] = $helper->sanitizeOutPut($metaData['license'], 'url');
                    }
                    if (!empty($metaData['acquireLicensePage'])) {
                        $image_license["acquireLicensePage"] = $helper->sanitizeOutPut($metaData['acquireLicensePage'], 'url');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_image_license', $image_license, $metaData));
                    break;

                case 'movie':
                    $movie = array();
                    $movie["@context"] = "http://schema.org";
                    $movie["@type"] = "Movie";
                    if (!empty($metaData['name'])) {
                        $movie["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $movie["description"] = $helper->sanitizeOutPut($metaData['description']);
                    }
                    if (!empty($metaData['duration'])) {
                        $movie["duration"] = $helper->sanitizeOutPut($metaData['duration']);
                    }
                    if (!empty($metaData['dateCreated'])) {
                        $movie["dateCreated"] = $helper->sanitizeOutPut($metaData['dateCreated']);
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $movie["image"] = $helper->sanitizeOutPut($img['url'], 'url');
                    }
                    if (!empty($metaData['director'])) {
                        $movie["director"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['director'])
                        );
                    }
                    if (!empty($metaData['author'])) {
                        $authorArray = explode("\r\n",
                            $helper->sanitizeOutPut($metaData['author'], 'textarea'));
                        $author = array();
                        if (!empty($authorArray) && is_array($authorArray) && count($authorArray)) {
                            foreach ($authorArray as $authorName) {
                                $author[] = array(
                                    "@type" => "Person",
                                    "name"  => $authorName
                                );
                            }
                        }
                        $movie["author"] = $author;
                    }
                    if (!empty($metaData['actor'])) {
                        $actorArray = explode("\r\n",
                            $helper->sanitizeOutPut($metaData['actor'], 'textarea'));
                        $actor = array();
                        if (!empty($actorArray) && is_array($actorArray) && count($actorArray)) {
                            foreach ($actorArray as $actorName) {
                                $actor[] = array(
                                    "@type" => "Person",
                                    "name"  => $actorName
                                );
                            }
                        }
                        $movie["actor"] = $actor;
                    }

                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $movie['image'] = $helper->sanitizeOutPut($img['url'], 'url');
                    }


                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_movie', $movie, $metaData));
                    if (isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $movie_review = array(
                            "@context" => "http://schema.org",
                            "@type"    => "Review",
                        );
                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $movie_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $movie_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }

                        unset($movie['@context']);
                        $movie['@type'] = "Movie";
                        if (isset($movie["description"])) {
                            $movie_review["description"] = Functions::filter_content($movie["description"], 200);
                            unset($movie["description"]);
                        }
                        if (isset($metaData['review_sameAs']) && !empty($metaData['review_sameAs'])) {
                            $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_sameAs'], "textarea"));
                            if (!empty($sameAs)) {
                                $movie["sameAs"] = $sameAs;
                            }
                        }

                        $movie_review["itemReviewed"] = $movie;
                        if (!empty($metaData['review_author'])) {
                            $movie_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $movie_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }
                        if (isset($metaData['review_publisher']) && !empty($metaData['review_publisher'])) {
                            $movie_review["publisher"] = array(
                                "@type" => "Organization",
                                "name"  => $helper->sanitizeOutPut($metaData['review_publisher'])
                            );
                            if (isset($metaData['review_publisherImage']) && !empty($metaData['review_publisherImage'])) {
                                $img = $helper->imageInfo(absint($metaData['review_publisherImage']));
                                $movie_review["review_publisher"]["logo"] = array(
                                    "@type"  => "ImageObject",
                                    "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                                    "height" => $img['height'],
                                    "width"  => $img['width']
                                );
                            }
                        }
                        if (isset($metaData['review_ratingValue'])) {
                            $movie_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $movie_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $movie_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }


                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_movie_review', $movie_review, $metaData));
                    }
                    break;

                case 'music':
                    $music = array();
                    $music["@context"] = "https://schema.org";
                    $music["@type"] = $helper->sanitizeOutPut($metaData['musicType']);
                    if (!empty($metaData['name'])) {
                        $music["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $music["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $movie["image"] = $helper->sanitizeOutPut($img['url'], 'url');
                    }
                    if (!empty($metaData['sameAs'])) {
                        $music["sameAs"] = $helper->sanitizeOutPut($metaData['sameAs'], 'url');
                    }
                    if (!empty($metaData['url'])) {
                        $music["url"] = $helper->sanitizeOutPut($metaData['url'], 'url');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_music', $music, $metaData));
                    break;

                case 'service':
                    $service = array(
                        "@context" => "https://schema.org",
                        "@type"    => "Service"
                    );
                    if (!empty($metaData['name'])) {
                        $service["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['serviceType'])) {
                        $service["serviceType"] = $helper->sanitizeOutPut($metaData['serviceType']);
                    }
                    if (!empty($metaData['award'])) {
                        $service["award"] = $helper->sanitizeOutPut($metaData['award']);
                    }
                    if (!empty($metaData['category'])) {
                        $service["category"] = $helper->sanitizeOutPut($metaData['category']);
                    }
                    if (!empty($metaData['providerMobility'])) {
                        $service["providerMobility"] = $helper->sanitizeOutPut($metaData['providerMobility']);
                    }
                    if (!empty($metaData['additionalType'])) {
                        $service["additionalType"] = $helper->sanitizeOutPut($metaData['additionalType']);
                    }
                    if (!empty($metaData['alternateName'])) {
                        $service["alternateName"] = $helper->sanitizeOutPut($metaData['alternateName']);
                    }
                    if (!empty($metaData['image'])) {
                        $service["image"] = $helper->sanitizeOutPut($metaData['image']);
                    }
                    if (!empty($metaData['mainEntityOfPage'])) {
                        $service["mainEntityOfPage"] = $helper->sanitizeOutPut($metaData['mainEntityOfPage']);
                    }
                    if (!empty($metaData['sameAs'])) {
                        $service["sameAs"] = $helper->sanitizeOutPut($metaData['sameAs']);
                    }
                    if (!empty($metaData['url'])) {
                        $service["url"] = $helper->sanitizeOutPut($metaData['url'], 'url');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_service', $service, $metaData));
                    break;

                case 'review':
                    $review = array(
                        "@context" => "https://schema.org",
                        "@type"    => "Review"
                    );
                    if (!empty($metaData['itemName'])) {
                        $review["itemReviewed"] = array(
                            "@type" => "product",
                            "name"  => $helper->sanitizeOutPut($metaData['itemName'])
                        );
                    }
                    if (!empty($metaData['ratingValue'])) {
                        $review["reviewRating"] = array(
                            "@type"       => "Rating",
                            "ratingValue" => $helper->sanitizeOutPut($metaData['ratingValue']),
                            "bestRating"  => $helper->sanitizeOutPut($metaData['bestRating']),
                            "worstRating" => $helper->sanitizeOutPut($metaData['worstRating'])
                        );
                    }
                    if (!empty($metaData['name'])) {
                        $review["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['author'])) {
                        $review["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($metaData['author'])
                        );
                    }
                    if (!empty($metaData['reviewBody'])) {
                        $review["reviewBody"] = $helper->sanitizeOutPut($metaData['reviewBody']);
                    }
                    if (!empty($metaData['datePublished'])) {
                        $review["datePublished"] = $helper->sanitizeOutPut($metaData['datePublished']);
                    }
                    if (!empty($metaData['publisher'])) {
                        $review["publisher"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['publisher'])
                        );
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_review', $review, $metaData));
                    break;

                case 'local_business':
                    $category = !empty($metaData['category']) ? $helper->sanitizeOutPut($metaData['category']) : "LocalBusiness";
                    $local_business = array(
                        "@context" => "https://schema.org",
                        "@type"    => $category
                    );

                    if (!empty($metaData['name'])) {
                        $local_business["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }

                    if (!empty($metaData['description'])) {
                        $local_business["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }

                    if (!empty($metaData['image'])) {
                        $imges = [];
                        foreach( $metaData['image'] as $img_single ) {
                            $img = $helper->imageInfo(absint($img_single));
                            $imges[] = $helper->sanitizeOutPut($img['url'], 'url');
                        }
                        $local_business["image"] = $imges;
                    }

                    if ( $category == 'Organization' && !empty($metaData['logo']) ) {
                        $img = $helper->imageInfo(absint($metaData['logo']));
                        $local_business["logo"] = $helper->sanitizeOutPut($img['url'], 'url');
                    }

                    if (!empty($metaData['priceRange'])) {
                        $local_business["priceRange"] = $helper->sanitizeOutPut($metaData['priceRange']);
                    } 

                    if ( $category == 'Restaurant' ) {
                        if (!empty($metaData['servesCuisine'])) {
                            $local_business["servesCuisine"] = $helper->sanitizeOutPut($metaData['servesCuisine']); 
                        }

                        if (!empty($metaData['menu'])) {
                            $local_business["menu"] = $helper->sanitizeOutPut($metaData['menu'], 'url');
                            $local_business["acceptsReservations"] = "True";
                        }
                    }

                    if (!empty($metaData['address'][0]['addressLocality']) || !empty($metaData['address'][0]['addressRegion'])
                        || !empty($metaData['address'][0]['postalCode']) || !empty($metaData['address'][0]['streetAddress'])) {
                        $local_business["address"] = array(
                            "@type"           => "PostalAddress",
                            "addressLocality" => $helper->sanitizeOutPut($metaData['address'][0]['addressLocality']),
                            "addressRegion"   => $helper->sanitizeOutPut($metaData['address'][0]['addressRegion']),
                            "postalCode"      => $helper->sanitizeOutPut($metaData['address'][0]['postalCode']),
                            "streetAddress"   => $helper->sanitizeOutPut($metaData['address'][0]['streetAddress']),
                            "addressCountry"  => $helper->sanitizeOutPut($metaData['address'][0]['addressCountry'])
                        );
                    }

                    if ( !empty($metaData['geo'][0]['latitude']) || !empty($metaData['geo'][0]['longitude']) ) {
                        $local_business["geo"] = array(
                            "@type"           => "GeoCoordinates",
                            "latitude" => $helper->sanitizeOutPut($metaData['geo'][0]['latitude']),
                            "longitude"   => $helper->sanitizeOutPut($metaData['geo'][0]['longitude']), 
                        );
                    }

                    if (!empty($metaData['telephone'])) {
                        $local_business["telephone"] = $helper->sanitizeOutPut($metaData['telephone']);
                    }

                    if (!empty($metaData['url'])) {
                        $local_business["url"] = $helper->sanitizeOutPut($metaData['url'], 'url');
                    }

                    if ( isset($metaData['opening_hours']) && is_array($metaData['opening_hours']) ) {
                        $local_business_opening_hours = array();
                        foreach ($metaData['opening_hours'] as $opening_hours_single) {
                            $opening_hours_single_schema = array(
                                "@type"          => "OpeningHoursSpecification",
                                "dayOfWeek"           => $opening_hours_single['day'] ? $helper->sanitizeOutPut($opening_hours_single['day']) : '', 
                                "opens"           => $opening_hours_single['opens'] ? $helper->sanitizeOutPut($opening_hours_single['opens']) : '', 
                                "closes"           => $opening_hours_single['closes'] ? $helper->sanitizeOutPut($opening_hours_single['closes']) : '', 
                            );
                            array_push($local_business_opening_hours, $opening_hours_single_schema); 
                        } 
                        $local_business['openingHoursSpecification'] = $local_business_opening_hours;
                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_local_business', $local_business, $metaData));
                    if ( isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $local_business_review = array(
                            "@context" => "https://schema.org",
                            "@type"    => "Review",
                        );
                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $local_business_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $local_business_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }

                        unset($local_business['@context']);
                        if (isset($local_business["description"])) {
                            $local_business_review["description"] = Functions::filter_content($local_business["description"], 200);
                            unset($local_business["description"]);
                        }
                        if (isset($metaData['review_sameAs']) && !empty($metaData['review_sameAs'])) {
                            $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_sameAs'], "textarea"));
                            if (!empty($sameAs)) {
                                $local_business["sameAs"] = $sameAs;
                            }
                        }

                        $local_business_review["itemReviewed"] = $local_business;
                        if (!empty($metaData['review_author'])) {
                            $local_business_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $local_business_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }
                        if (isset($metaData['review_ratingValue'])) {
                            $local_business_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $local_business_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $local_business_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }
                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_local_business_review', $local_business_review, $metaData));
                    }
                    break;
                
                case 'software_app':
                    $app = array(
                        "@context" => "http://schema.org",
                        "@type"    => "SoftwareApplication"
                    );
                    if (!empty($metaData['name'])) {
                        $app["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $app["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }

                    if (!empty($metaData['applicationCategory'])) {
                        $app["applicationCategory"] = $helper->sanitizeOutPut($metaData['applicationCategory']);
                    }

                    if (!empty($metaData['operatingSystem'])) {
                        $app["operatingSystem"] = $helper->sanitizeOutPut($metaData['operatingSystem']);
                    }

                    if (!empty($metaData['price'])) {
                        $app["offers"] = array(
                            "@type" => 'Offer',
                            "price" => $helper->sanitizeOutPut($metaData['price'])
                        );
                        if (!empty($metaData['priceCurrency'])) {
                            $app["offers"]["priceCurrency"] = $helper->sanitizeOutPut($metaData['priceCurrency']);
                        }
                    }
                    if (isset($metaData['aggregate_ratingValue'])) {
                        $app["aggregateRating"] = array(
                            "@type"       => 'AggregateRating',
                            "ratingValue" => $helper->sanitizeOutPut($metaData['aggregate_ratingValue'], "number")
                        );
                        if (isset($metaData['aggregate_bestRating'])) {
                            $app["aggregateRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['aggregate_bestRating'], "number");
                        }
                        if (isset($metaData['aggregate_worstRating'])) {
                            $app["aggregateRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['aggregate_worstRating'], "number");
                        }
                        if (isset($metaData['aggregate_ratingCount'])) {
                            $app["aggregateRating"]["reviewCount"] = $helper->sanitizeOutPut($metaData['aggregate_ratingCount'], "number");
                        }
                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_software_application', $app, $metaData));
                    if ( isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $app_review = array(
                            "@context" => "http://schema.org",
                            "@type"    => "Review",
                        );
                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $app_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $app_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }

                        unset($app['@context']);
                        if (isset($app["description"])) {
                            $app_review["description"] = Functions::filter_content($app["description"], 200);
                            unset($app["description"]);
                        }
                        if (isset($metaData['review_sameAs']) && !empty($metaData['review_sameAs'])) {
                            $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_sameAs'], "textarea"));
                            if (!empty($sameAs)) {
                                $app["sameAs"] = $sameAs;
                            }
                        }

                        $app_review["itemReviewed"] = $app;
                        if (!empty($metaData['review_author'])) {
                            $app_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $app_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }
                        if (isset($metaData['review_ratingValue'])) {
                            $app_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $app_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $app_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }


                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_software_application_review', $app_review, $metaData));
                    }
                    break;

                case 'job_posting':
                    $jobPosting = array();
                    $jobPosting["@context"] = "http://schema.org";
                    $jobPosting["@type"] = "JobPosting";
                    if (!empty($metaData['title'])) {
                        $jobPosting["title"] = $helper->sanitizeOutPut($metaData['title']);
                    }
                    if (!empty($metaData['workHours'])) {
                        $jobPosting["workHours"] = $helper->sanitizeOutPut($metaData['workHours']);
                    }
                    if (!empty($metaData['salaryAmount']) || !empty($metaData['currency']) || !empty($metaData['salaryAt'])) {

                        $jobPosting["baseSalary"] = array(
                            "@type"    => "MonetaryAmount",
                            "currency" => $helper->sanitizeOutPut($metaData['currency']),
                            "value"    => array(
                                "@type"    => "QuantitativeValue",
                                "value"    => $helper->sanitizeOutPut($metaData['salaryAmount']),
                                "unitText" => $helper->sanitizeOutPut($metaData['salaryAt'])
                            )
                        );
                    }
                    if (!empty($metaData['jobBenefits'])) {
                        $jobPosting["jobBenefits"] = $helper->sanitizeOutPut($metaData['jobBenefits']);
                    }
                    if (!empty($metaData['datePosted'])) {
                        $jobPosting["datePosted"] = $helper->sanitizeOutPut($metaData['datePosted']);
                    }
                    if (!empty($metaData['validThrough'])) {
                        $jobPosting["validThrough"] = $helper->sanitizeOutPut($metaData['validThrough']);
                    }
                    if (!empty($metaData['description'])) {
                        $jobPosting["description"] = $helper->sanitizeOutPut($metaData['description'],
                            'textarea');
                    }
                    if (!empty($metaData['educationRequirements'])) {
                        $jobPosting["educationRequirements"] = [
                            '@type' => 'EducationalOccupationalCredential',
                            'credentialCategory' => $helper->sanitizeOutPut($metaData['educationRequirements'])
                        ];
                    } 
                    if (!empty($metaData['employmentType'])) {
                        $jobPosting["employmentType"] = $helper->sanitizeOutPut($metaData['employmentType']);
                    } 
                    if (!empty($metaData['experienceRequirements'])) {
                        $jobPosting["experienceRequirements"] = [
                            '@type' => 'OccupationalExperienceRequirements',
                            'monthsOfExperience' => $helper->sanitizeOutPut($metaData['experienceRequirements'])
                        ];
                    } 
                    if (!empty($metaData['incentiveCompensation'])) {
                        $jobPosting["incentiveCompensation"] = $helper->sanitizeOutPut($metaData['incentiveCompensation']);
                    }
                    if (!empty($metaData['hiringOrganization'])) {
                        $jobPosting["hiringOrganization"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['hiringOrganization'])
                        );
                    }
                    if (!empty($metaData['industry'])) {
                        $jobPosting["industry"] = $helper->sanitizeOutPut($metaData['industry']);
                    }
                    if (!empty($metaData['addressLocality']) || !empty($metaData['addressRegion'])) {
                        $jobPosting["jobLocation"] = array(
                            "@type"   => "Place",
                            "address" => array(
                                "@type"           => "PostalAddress",
                                "addressLocality" => $helper->sanitizeOutPut($metaData['addressLocality']),
                                "addressRegion"   => $helper->sanitizeOutPut($metaData['addressRegion']),
                                "postalCode"      => $helper->sanitizeOutPut($metaData['postalCode']),
                                "streetAddress"   => $helper->sanitizeOutPut($metaData['streetAddress'])
                            )
                        );
                    }
                    if (!empty($metaData['occupationalCategory'])) {
                        $jobPosting["occupationalCategory"] = $helper->sanitizeOutPut($metaData['occupationalCategory']);
                    }
                    if (!empty($metaData['qualifications'])) {
                        $jobPosting["qualifications"] = $helper->sanitizeOutPut($metaData['qualifications'],
                            'textarea');
                    }
                    if (!empty($metaData['responsibilities'])) {
                        $jobPosting["responsibilities"] = $helper->sanitizeOutPut($metaData['responsibilities'],
                            'textarea');
                    }
                    if (!empty($metaData['salaryCurrency'])) {
                        $jobPosting["salaryCurrency"] = $helper->sanitizeOutPut($metaData['salaryCurrency']);
                    }
                    if (!empty($metaData['skills'])) {
                        $jobPosting["skills"] = $helper->sanitizeOutPut($metaData['skills'], 'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_job_posting', $jobPosting, $metaData));
                    break;

                case 'faq':
                    $faqSchema = array(
                        "@context" => "http://schema.org",
                        "@type"    => "FAQPage"
                    );

                    if ( isset($metaData['faqs']) && is_array($metaData['faqs']) ) {
                        $faqs_schema = array();
                        foreach ($metaData['faqs'] as $position => $faq_item) {
                            $faq_item_schema = array(
                                "@type"          => "Question",
                                "name"           => $faq_item['ques'] ? $helper->sanitizeOutPut($faq_item['ques']) : null,
                                "acceptedAnswer" => array(
                                    "@type" => "Answer",
                                    "text"  => isset($faq_item['ans']) ? $helper->sanitizeOutPut($faq_item['ans'], 'textarea') : null
                                )
                            );
                            array_push($faqs_schema, $faq_item_schema);

                        }
                        if (count($faqs_schema) == 1) {
                            $faqSchema['mainEntity'] = $faqs_schema[0];
                        } else {
                            $faqSchema['mainEntity'] = $faqs_schema;
                        }
                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_faq', $faqSchema, $metaData));
                    break;

                case 'question_answer':
                    $questionAnswerSchema = array(
                        "@context" => "https://schema.org",
                        "@type"    => "QAPage"
                    );

                    $question = array(
                        "@type"    => "Question"
                    );
                    if (!empty($metaData['name'])) {
                        $question["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['text'])) {
                        $question["text"] = $helper->sanitizeOutPut($metaData['text'],
                        'textarea');
                    }
                    if (!empty($metaData['answerCount'])) {
                        $question["answerCount"] = $helper->sanitizeOutPut($metaData['answerCount']);
                    }
                    if (!empty($metaData['upvoteCount'])) {
                        $question["upvoteCount"] = $helper->sanitizeOutPut($metaData['upvoteCount']);
                    }
                    if (!empty($metaData['dateCreated'])) {
                        $question["dateCreated"] = $helper->sanitizeOutPut($metaData['dateCreated']);
                    }
                    if (!empty($metaData['author'])) {
                        $question["author"] = [
                            '@type' => 'Person',
                            'name' => $helper->sanitizeOutPut($metaData['author']),
                        ];
                    }

                    if ( isset($metaData['answers']) && is_array($metaData['answers']) ) {
                        $acceptedAnswer = $suggestedAnswer = array();
                        foreach ($metaData['answers'] as $position => $answer_item) {
                            $answer_item_schema = array(
                                "@type"          => "Answer",
                                "text"           => $answer_item['text'] ? $helper->sanitizeOutPut($answer_item['text']) : null,

                                "dateCreated"   => $answer_item['dateCreated'] ? $helper->sanitizeOutPut($answer_item['dateCreated']) : null,

                                "upvoteCount"   => $answer_item['upvoteCount'] ? $helper->sanitizeOutPut($answer_item['upvoteCount']) : 0,
                                
                                "url"           => $answer_item['url'] ? $helper->sanitizeOutPut($answer_item['url'], 'url') : null,

                                "author" => array(
                                    "@type" => "Person",
                                    "author"  => isset($answer_item['author']) ? $helper->sanitizeOutPut($answer_item['author']) : null
                                )
                            );

                            if ( $answer_item['answerType'] == 'normal' ) {
                                array_push($suggestedAnswer, $answer_item_schema);
                            } else {
                                $acceptedAnswer = $answer_item_schema;
                            }

                            if ( $acceptedAnswer ) {
                                $question["acceptedAnswer"] = $acceptedAnswer;
                            }

                            if ( $suggestedAnswer ) {
                                $question["suggestedAnswer"] = $suggestedAnswer;
                            }
                        } 
                    }

                    $questionAnswerSchema["mainEntity"] = $question;

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_faq', $questionAnswerSchema, $metaData));
                    break;

                case 'how_to':
                    $howToSchema = array(
                        "@context" => "https://schema.org",
                        "@type"    => "HowTo"
                    );

                    if (!empty($metaData['name'])) {
                        $howToSchema["name"] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['description'])) {
                        $howToSchema["description"] = $helper->sanitizeOutPut($metaData['description'],
                        'textarea');
                    }

                    if (!empty($metaData['image'])) {
                        $img = $helper->imageInfo(absint($metaData['image']));
                        $howToSchema["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }

                    if ( !empty($metaData['price']) ) {
                        $howToSchema["estimatedCost"] = array(
                            "@type" => "MonetaryAmount",
                            "value" => $helper->sanitizeOutPut($metaData['price'])
                        );
                        if (!empty($metaData['priceCurrency'])) {
                            $howToSchema["estimatedCost"]['currency'] = $helper->sanitizeOutPut($metaData['priceCurrency']);
                        }
                    }

                    if ( isset($metaData['supply']) && is_array($metaData['supply']) ) {
                        $how_to_supply = array();
                        foreach ($metaData['supply'] as $supply_single) {
                            $supply_single_schema = array(
                                "@type"          => "HowToSupply",
                                "name"           => $supply_single['name'] ? $helper->sanitizeOutPut($supply_single['name']) : null, 
                            );
                            array_push($how_to_supply, $supply_single_schema);

                        } 
                        $howToSchema['supply'] = $how_to_supply;
                    }

                    if ( isset($metaData['tool']) && is_array($metaData['tool']) ) {
                        $how_to_tool = array();
                        foreach ($metaData['tool'] as $tool_single) {
                            $tool_single_schema = array(
                                "@type"   => "HowToTool",
                                "name"    => $tool_single['name'] ? $helper->sanitizeOutPut($tool_single['name']) : null, 
                            );
                            array_push($how_to_tool, $tool_single_schema); 
                        } 
                        $howToSchema['tool'] = $how_to_tool;
                    }

                    if ( isset($metaData['step']) && is_array($metaData['step']) ) {
                        $how_to_step = array();
                        foreach ($metaData['step'] as $step_single) {
                            $step_single_schema = array(
                                "@type"   => "HowToStep", 
                            );

                            if ( !empty( $step_single['name'] ) ) {
                                $step_single_schema["name"] = $helper->sanitizeOutPut($step_single['name']);
                            }

                            if ( !empty( $step_single['text'] ) ) {
                                $step_single_schema["text"] =  $helper->sanitizeOutPut($step_single['text'], 'textarea');
                            }

                            if ( !empty( $step_single['url'] ) ) {
                                $step_single_schema["url"] = $helper->sanitizeOutPut($step_single['url'], 'url');
                            }

                            if ( !empty( $step_single['image'] ) ) {
                                $img = $helper->imageInfo(absint($step_single['image']));
                                $step_single_schema["image"] = array(
                                    "@type"  => "ImageObject",
                                    "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                                    "height" => $img['height'],
                                    "width"  => $img['width']
                                );
                            }

                            if ( !empty( $step_single['clipId'] ) ) { 
                                $step_single_schema["video"] = array(
                                    "@id"  => $helper->sanitizeOutPut( $step_single['clipId'] ),
                                );
                            }

                            if ( isset($step_single['direction']) && is_array($step_single['direction']) ) {
                                $how_to_step_direction = array();
                                foreach ($step_single['direction'] as $direction_single) {
                                    $direction_single_schema = array(
                                        "@type"          => "HowToDirection",
                                        "text"           => $direction_single['text'] ? $helper->sanitizeOutPut($direction_single['text']) : null, 
                                    );
                                    array_push($how_to_step_direction, $direction_single_schema); 
                                } 
                                $step_single_schema['itemListElement'] = $how_to_step_direction;
                            }

                            array_push($how_to_step, $step_single_schema); 
                        } 
                        $howToSchema['step'] = $how_to_step;
                    }

                    if ( isset($metaData['video']) && is_array($metaData['video']) ) {
                        $how_to_video = array();
                        foreach ($metaData['video'] as $video_single) {
                            if ( $video_single['name'] && $video_single['contentUrl'] ) {
                                $video_single_schema = array(
                                    "@type"         => "VideoObject",
                                    "name"          => $video_single['name'] ? $helper->sanitizeOutPut($video_single['name']) : null, 
                                    "description"   => $video_single['description'] ? $helper->sanitizeOutPut($video_single['description']) : null,  
                                    "contentUrl"    => $video_single['contentUrl'] ? $helper->sanitizeOutPut($video_single['contentUrl']) : null, 
                                    "embedUrl"      => $video_single['embedUrl'] ? $helper->sanitizeOutPut($video_single['embedUrl']) : null, 
                                    "uploadDate"    => $video_single['uploadDate'] ? $helper->sanitizeOutPut($video_single['uploadDate']) : null, 
                                    "duration"      => $video_single['duration'] ? $helper->sanitizeOutPut($video_single['duration']) : null, 
                                );
                                if (!empty($video_single['thumbnailUrl'])) {
                                    $img = $helper->imageInfo(absint($video_single['thumbnailUrl']));
                                    $video_single_schema["thumbnailUrl"] = $helper->sanitizeOutPut($img['url'], 'url');
                                }
    
                                $how_to_video = $video_single_schema;

                                if ( isset($video_single['clip']) && is_array($video_single['clip']) ) {
                                    $how_to_video_clip = array();
                                    foreach ($video_single['clip'] as $clip_single) {
                                        $clip_single_schema = array(
                                            "@type"  => "Clip", 
                                            "@id"   => $clip_single['id'] ? $helper->sanitizeOutPut($clip_single['id']) : '', 
                                            "name"   => $clip_single['name'] ? $helper->sanitizeOutPut($clip_single['name']) : null, 
                                            "startOffset"   => $clip_single['startOffset'] ? $helper->sanitizeOutPut($clip_single['startOffset']) : null, 
                                            "endOffset"   => $clip_single['endOffset'] ? $helper->sanitizeOutPut($clip_single['endOffset']) : null, 
                                            "endOffset"   => $clip_single['endOffset'] ? $helper->sanitizeOutPut($clip_single['endOffset']) : null, 
                                            "url"   => $clip_single['url'] ? $helper->sanitizeOutPut($clip_single['url'], 'url') : null, 
                                        );
                                        array_push($how_to_video_clip, $clip_single_schema); 
                                    } 
                                    $how_to_video['hasPart'] = $how_to_video_clip;
                                }
                            } 
                        } 
                        if ( $how_to_video ) {
                            $howToSchema['video'] = $how_to_video;
                        }
                    }

                    if (!empty($metaData['totalTime'])) {
                        $howToSchema["totalTime"] = $helper->sanitizeOutPut($metaData['totalTime']);
                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_faq', $howToSchema, $metaData));
                    break;

                case 'breadcrumb':
                    $breadcrumbSchema = array(
                        "@context" => "http://schema.org",
                        "@type"    => "BreadcrumbList"
                    );

                    if ( isset($metaData['items']) && is_array($metaData['items']) ) {
                        $breadcrumbs_schema = array();
                        foreach ($metaData['items'] as $position => $breadcrumb_item) {
                            $breadcrumb_item_schema = array(
                                "@type"          => "ListItem",
                                "position"       => $position+1,
                                "name"           => $breadcrumb_item['name'] ? $helper->sanitizeOutPut($breadcrumb_item['name']) : '',
                                "item"           => $breadcrumb_item['item'] ? $helper->sanitizeOutPut($breadcrumb_item['item']) : '',
                            
                            );
                            array_push($breadcrumbs_schema, $breadcrumb_item_schema);

                        }
                        
                        $breadcrumbSchema['itemListElement'] = $breadcrumbs_schema;

                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_breadcrumb', $breadcrumbSchema, $metaData));
                    break;

                case 'itemlist':
                    $itemlistSchema = array(
                        "@context" => "http://schema.org",
                        "@type"    => "ItemList"
                    );

                    if ( isset($metaData['items']) && is_array($metaData['items']) ) {
                        $itemlists_schema = array();
                        foreach ($metaData['items'] as $position => $itemlist_item) {
                            $itemlist_item_schema = array(
                                "@type"          => "ListItem",
                                "position"       => $position+1, 
                                "url"           => $itemlist_item['url'] ? $helper->sanitizeOutPut($itemlist_item['url']) : '',
                            
                            );
                            array_push($itemlists_schema, $itemlist_item_schema);

                        }
                        
                        $itemlistSchema['itemListElement'] = $itemlists_schema;

                    }

                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_itemlist', $itemlistSchema, $metaData));
                    break;
                
                case 'special_announcement':
                    $announcement = array(
                        "@context" => "https://schema.org",
                        "@type"    => "SpecialAnnouncement",
                        "category" => "https://www.wikidata.org/wiki/Q81068910"
                    );
                    if (!empty($metaData['name'])) {
                        $announcement['name'] = $helper->sanitizeOutPut($metaData['name']);
                    }
                    if (!empty($metaData['datePublished'])) {
                        $announcement['datePosted'] = $helper->sanitizeOutPut($metaData['datePublished']);
                    }
                    if (!empty($metaData['expires'])) {
                        $announcement['expires'] = $helper->sanitizeOutPut($metaData['expires']);
                    }
                    if (!empty($metaData['text'])) {
                        $announcement['text'] = $helper->sanitizeOutPut($metaData['text'], 'textarea');
                    }
                    if (!empty($metaData['url'])) {
                        $announcement['url'] = $helper->sanitizeOutPut($metaData['url'], 'url');
                    }
                    if (isset($metaData['locations']) && is_array($metaData['locations']) && !empty($metaData['locations'])) {
                        $locations_schema = [];
                        foreach ($metaData['locations'] as $position => $location) {
                            if ($location['type']) {
                                $location_schema = array(
                                    "@type"   => $helper->sanitizeOutPut($location['type']),
                                    'name'    => !empty($location['name']) ? $helper->sanitizeOutPut($location['name']) : "",
                                    'url'     => !empty($location['url']) ? $helper->sanitizeOutPut($location['url'], 'url') : '',
                                    "address" => [
                                        "@type" => "PostalAddress",
                                    ]
                                );
                                if (!empty($location['id'])) {
                                    $location_schema['@id'] = $helper->sanitizeOutPut($location['id']);
                                }
                                if (!empty($location['image'])) {
                                    $img = $helper->imageInfo(absint($location['image']));
                                    $location_schema["image"] = $helper->sanitizeOutPut($img['url'], 'url');
                                }
                                if (!empty($location['url'])) {
                                    $location_schema['url'] = $helper->sanitizeOutPut($location['url'], 'url');
                                }
                                if (!empty($location['address_street'])) {
                                    $location_schema['address']['streetAddress'] = $helper->sanitizeOutPut($location['address_street']);
                                }
                                if (!empty($location['address_locality'])) {
                                    $location_schema['address']['addressLocality'] = $helper->sanitizeOutPut($location['address_locality']);
                                }
                                if (!empty($location['address_post_code'])) {
                                    $location_schema['address']['postalCode'] = $helper->sanitizeOutPut($location['address_post_code']);
                                }
                                if (!empty($location['address_region'])) {
                                    $location_schema['address']['addressRegion'] = $helper->sanitizeOutPut($location['address_region']);
                                }
                                if (!empty($location['address_country'])) {
                                    $location_schema['address']['addressCountry'] = $helper->sanitizeOutPut($location['address_country']);
                                }
                                if (!empty($location['priceRange'])) {
                                    $location_schema["priceRange"] = $helper->sanitizeOutPut($location['priceRange']);
                                }
                                if (!empty($location['telephone'])) {
                                    $location_schema["telephone"] = $helper->sanitizeOutPut($location['telephone']);
                                }
                                array_push($locations_schema, $location_schema);
                            }

                        }
                        if (count($locations_schema) === 1) {
                            $announcement['announcementLocation'] = $locations_schema[0];
                        } else {
                            $announcement['announcementLocation'] = $locations_schema;
                        }
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_item_list', $announcement, $metaData));
                    break;
                default:
                    break;
            }
        }
        return $html; 
    }
    
    /**
     * schema output
     * @return object
     */
    public function autoSchemaOutput( $schemaCat, $post_id = null) { 
        $html = null;
        $metaData = [];
        if ( $schemaCat ) {  
            $helper = new Functions();
            $post   = get_post( $post_id );

            $publisher = rtrs()->get_options('rtrs_schema_publisher_settings', 'publisher_name'); 
            if ( $publisher ) {
                $metaData['publisher'] = $publisher;
            }
            $publisher_logo = rtrs()->get_options('rtrs_schema_publisher_settings', 'publisher_logo');  
            if ( $publisher_logo ) {
                $metaData['publisherImage'] = $publisher_logo;
            }

            switch ( $schemaCat ) {
                case "article": 
                    $article = array(
                        "@context" => "https://schema.org",
                        "@type"    => "Article"
                    );
                    if (!empty( $post->post_title )) {
                        $article["headline"] = $helper->sanitizeOutPut( $post->post_title );
                    }
                    if ( !empty( $post_url = get_the_permalink( $post_id ) ) ) {
                        $article["mainEntityOfPage"] = array(
                            "@type" => "WebPage",
                            "@id"   => $helper->sanitizeOutPut($post_url)
                        );
                    }
                    if (!empty($author = get_the_author_meta('display_name', $post->post_author) )) {
                        $article["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($author)
                        );

                        if ( !empty($url = get_the_author_meta('url', $post->post_author)) ) {
                            $article["author"]['url'] = $helper->sanitizeOutPut($url, 'url');
                        } 
                    }
                    if (!empty($metaData['publisher'])) {
                        if (!empty($metaData['publisherImage'])) {
                            $img = $helper->imageInfo(absint($metaData['publisherImage']));
                            $plA = array(
                                "@type"  => "ImageObject",
                                "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                                "height" => $img['height'],
                                "width"  => $img['width']
                            );
                        } else {
                            $plA = array();
                        }
                        $article["publisher"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['publisher']),
                            "logo"  => $plA
                        );
                    }
                    if (!empty( $image_id = get_post_thumbnail_id( $post->ID ) )) {
                        $img = $helper->imageInfo(absint( $image_id ));
                        $article["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }
                    if (!empty( $post->post_date )) {
                        $article["datePublished"] = $helper->sanitizeOutPut($post->post_date);
                    }
                    if (!empty( $post->post_modified )) {
                        $article["dateModified"] = $helper->sanitizeOutPut($post->post_modified);
                    }
                    if (!empty( $post->post_excerpt )) {
                        $article["description"] = $helper->sanitizeOutPut($post->post_excerpt,
                            'textarea');
                    }
                    if (!empty( $post->post_content )) {
                        $article["articleBody"] = $helper->sanitizeOutPut($post->post_content,
                            'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_article', $article, $metaData));
                    break;

                case "news_article":
                    $newsArticle = array(
                        "@context" => "https://schema.org",
                        "@type"    => "NewsArticle"
                    );
                    if (!empty( $post->post_title )) {
                        $newsArticle["headline"] = $helper->sanitizeOutPut( $post->post_title );
                    }
                    if ( !empty( $post_url = get_the_permalink( $post_id ) ) ) {
                        $newsArticle["mainEntityOfPage"] = array(
                            "@type" => "WebPage",
                            "@id"   => $helper->sanitizeOutPut($post_url)
                        );
                    }
                    if (!empty($author = get_the_author_meta('display_name', $post->post_author) )) {
                        $newsArticle["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($author)
                        );

                        if ( !empty($url = get_the_author_meta('url', $post->post_author)) ) {
                            $newsArticle["author"]['url'] = $helper->sanitizeOutPut($url, 'url');
                        }
                    }
                    if (!empty( $image_id = get_post_thumbnail_id( $post->ID ) )) {
                        $img = $helper->imageInfo(absint( $image_id ));
                        $newsArticle["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }
                    if (!empty( $post->post_date )) {
                        $newsArticle["datePublished"] = $helper->sanitizeOutPut($post->post_date);
                    }
                    if (!empty( $post->post_modified )) {
                        $newsArticle["dateModified"] = $helper->sanitizeOutPut($post->post_modified);
                    }
                    if (!empty( $post->post_excerpt )) {
                        $newsArticle["description"] = $helper->sanitizeOutPut($post->post_excerpt,
                            'textarea');
                    }
                    if (!empty( $post->post_content )) {
                        $newsArticle["articleBody"] = $helper->sanitizeOutPut($post->post_content,
                            'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_news_article', $newsArticle, $metaData));
                    break;

                case "blog_posting":
                    $blogPosting = array(
                        "@context" => "https://schema.org",
                        "@type"    => "BlogPosting"
                    );
                    if (!empty( $post->post_title )) {
                        $blogPosting["headline"] = $helper->sanitizeOutPut( $post->post_title );
                    }
                    if ( !empty( $post_url = get_the_permalink( $post_id ) ) ) {
                        $blogPosting["mainEntityOfPage"] = array(
                            "@type" => "WebPage",
                            "@id"   => $helper->sanitizeOutPut($post_url)
                        );
                    }
                    if (!empty($author = get_the_author_meta('display_name', $post->post_author) )) {
                        $blogPosting["author"] = array(
                            "@type" => "Person",
                            "name"  => $helper->sanitizeOutPut($author)
                        );

                        if ( !empty($url = get_the_author_meta('url', $post->post_author)) ) {
                            $blogPosting["author"]['url'] = $helper->sanitizeOutPut($url, 'url');
                        } 
                    }
                    if (!empty( $image_id = get_post_thumbnail_id( $post->ID ) )) {
                        $img = $helper->imageInfo(absint( $image_id ));
                        $blogPosting["image"] = array(
                            "@type"  => "ImageObject",
                            "url"    => $helper->sanitizeOutPut($img['url'], 'url'),
                            "height" => $img['height'],
                            "width"  => $img['width']
                        );
                    }
                    if (!empty( $post->post_date )) {
                        $blogPosting["datePublished"] = $helper->sanitizeOutPut($post->post_date);
                    }
                    if (!empty( $post->post_modified )) {
                        $blogPosting["dateModified"] = $helper->sanitizeOutPut($post->post_modified);
                    }
                    if (!empty( $post->post_excerpt )) {
                        $blogPosting["description"] = $helper->sanitizeOutPut($post->post_excerpt,
                            'textarea');
                    }
                    if (!empty( $post->post_content )) {
                        $blogPosting["articleBody"] = $helper->sanitizeOutPut($post->post_content,
                            'textarea');
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_blog_posting', $blogPosting, $metaData));
                    break;

                case "course":

                    if ( !function_exists('rtrsp') ) break;
                    if ( $post->post_type != 'lp_course' || !is_plugin_active('learnpress/learnpress.php') ) break;

                    $post_meta = get_post_meta($post->ID);
                    $price = isset( $post_meta['_lp_price'] ) ? $post_meta['_lp_price'][0] : '';
                    $sale_price = isset( $post_meta['_lp_sale_price'] ) ? $post_meta['_lp_sale_price'][0] : '';
                    
                    if ( $sale_price ) {
                        $price = $sale_price;
                    }
                    
                    if ( $price ) {
                        $metaData['price'] = $price;
                        $metaData['priceCurrency'] = rtrs()->get_options('learn_press_currency');
                        $metaData['availability'] = 'http://schema.org/InStock';
                        $metaData['url'] = get_the_permalink( $post->ID );
                    }
                    $metaData['courseMode'] = 'online';
                     
                    $course = array();
                    $course["@context"] = "http://schema.org";
                    $course["@type"] = "Course";
                    $course["hasCourseInstance"] = array(
                        "@type" => "CourseInstance"
                    );

                    if (!empty($post->post_title)) {
                        $course["name"] = $course["hasCourseInstance"]["name"] = $helper->sanitizeOutPut($post->post_title);
                    }
                    if (!empty( $post->post_content )) {
                        $course["description"] = $course["hasCourseInstance"]["description"] = $helper->sanitizeOutPut( $post->post_content );
                    }
                    if (!empty($metaData['publisher'])) {
                        $course["provider"] = array(
                            "@type" => "Organization",
                            "name"  => $helper->sanitizeOutPut($metaData['publisher'])
                        );
                    }

                    if (!empty($metaData['courseMode'])) {
                        $course["hasCourseInstance"]["courseMode"] = explode("\r\n",
                            $helper->sanitizeOutPut($metaData['courseMode'], 'textarea'));

                    }
                    if (!empty($metaData['endDate'])) {
                        $course["hasCourseInstance"]["endDate"] = $helper->sanitizeOutPut($metaData['endDate']);
                    }
                    if (!empty($metaData['startDate'])) {
                        $course["hasCourseInstance"]["startDate"] = $helper->sanitizeOutPut($metaData['startDate']);
                    }
                    if (!empty($metaData['locationName']) && !empty($metaData['locationAddress'])) {
                        $course["hasCourseInstance"]["location"] = array(
                            "@type"   => "Place",
                            "name"    => $helper->sanitizeOutPut($metaData['locationName']),
                            "address" => $helper->sanitizeOutPut($metaData['locationAddress'])
                        );
                    }
                    if ( $img_url = get_the_post_thumbnail_url( $post->ID ) ) {
                        $course["hasCourseInstance"]["image"] = $helper->sanitizeOutPut($img_url,
                            'url');
                    }
                    if (!empty($metaData['price']) && !empty($metaData['priceCurrency'])) {
                        $course["hasCourseInstance"]["offers"] = array(
                            "@type"         => 'Offer',
                            "price"         => $helper->sanitizeOutPut($metaData['price']),
                            "priceCurrency" => $helper->sanitizeOutPut($metaData['priceCurrency']),
                        );
                        if (!empty($metaData['availability'])) {
                            $course["hasCourseInstance"]["offers"]["availability"] = $helper->sanitizeOutPut($metaData['availability']);
                        }
                        if (!empty($metaData['url'])) {
                            $course["hasCourseInstance"]["offers"]["url"] = $helper->sanitizeOutPut($metaData['url']);
                        }
                        if (!empty($metaData['validFrom'])) {
                            $course["hasCourseInstance"]["offers"]["validFrom"] = $helper->sanitizeOutPut($metaData['validFrom']);
                        }
                    }
                    if (!empty($metaData['performerType']) && !empty($metaData['performerName'])) {
                        $course["hasCourseInstance"]["performer"] = array(
                            "@type" => $helper->sanitizeOutPut($metaData['performerType']),
                            "name"  => $helper->sanitizeOutPut($metaData['performerName'])
                        );
                    }
                    $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_course', $course, $metaData));
                    if ( isset($metaData['review_active']) && $metaData['review_active'] == 'show' ) {
                        $course_review = array(
                            "@context"     => "http://schema.org",
                            "@type"        => "Review",
                            "itemReviewed" => array(
                                "@type" => "Course",
                            )
                        );

                        if (isset($metaData['review_datePublished']) && !empty($metaData['review_datePublished'])) {
                            $course_review["datePublished"] = $helper->sanitizeOutPut($metaData['review_datePublished']);
                        }
                        if (isset($metaData['review_body']) && !empty($metaData['review_body'])) {
                            $course_review["reviewBody"] = $helper->sanitizeOutPut($metaData['review_body'], 'textarea');
                        }
                        if (isset($course["name"])) {
                            $course_review["itemReviewed"]['name'] = $course["name"];
                        }

                        if (isset($course["description"])) {
                            $course_review["itemReviewed"]["description"] = Functions::filter_content($course["description"], 200);
                        }
                        if (isset($course["provider"])) {
                            $course_review["itemReviewed"]['provider'] = $course["provider"];
                        }
                        if (!empty($metaData['review_author'])) {
                            $course_review["author"] = array(
                                "@type" => "Person",
                                "name"  => $helper->sanitizeOutPut($metaData['review_author'])
                            );

                            if (isset($metaData['review_author_sameAs']) && !empty($metaData['review_author_sameAs'])) {
                                $sameAs = Functions::get_same_as($helper->sanitizeOutPut($metaData['review_author_sameAs'], "textarea"));
                                if (!empty($sameAs)) {
                                    $course_review["author"]["sameAs"] = $sameAs;
                                }
                            }
                        }
                        if (isset($metaData['review_ratingValue'])) {
                            $course_review["reviewRating"] = array(
                                "@type"       => "Rating",
                                "ratingValue" => $helper->sanitizeOutPut($metaData['review_ratingValue'], 'number')
                            );
                            if (isset($metaData['review_bestRating'])) {
                                $course_review["reviewRating"]["bestRating"] = $helper->sanitizeOutPut($metaData['review_bestRating'], 'number');
                            }
                            if (isset($metaData['review_worstRating'])) {
                                $course_review["reviewRating"]["worstRating"] = $helper->sanitizeOutPut($metaData['review_worstRating'], 'number');
                            }
                        }
                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_course_review', $course_review, $metaData)); 
                    }
                    break;

                case 'product':
                    
                    if ( !function_exists('rtrsp') ) break;
                    $markup = [];

                    //if edd
                    if ( $post->post_type == 'download' && function_exists('EDD') ) {
                        $permalink = get_the_permalink( $post->ID );
                        $markup = array(
                            "@context" => "https://schema.org",
                            '@type'       => 'Product',
                            '@id'         => $permalink . '#comments', // Append '#comments' to differentiate between this @id and the @id generated for the Breadcrumblist.
                            'name'        => wp_kses_post( $post->post_title ),
                            'url'         => $permalink,
                            'description' => wp_strip_all_tags( do_shortcode( $post->post_excerpt ? $post->post_excerpt : $post->post_content ) ),
                        );

                        $image = get_the_post_thumbnail_url( $post->ID ); 
                        if ( $image ) {
                            $markup['image'] = $image;
                        }

                        $markup['sku'] = $post->ID;
                        $brand_name = rtrs()->get_options('rtrs_woocommerce_settings', 'brand_name');  
                        if ( $brand_name ) {
                            $markup['brand'] = [
                                "@type" => "Brand",
                                "name" => esc_html( $brand_name )
                            ];
                        }

                        $identifier_type = rtrs()->get_options('rtrs_woocommerce_settings', 'identifier_type');  
                        $identifier = rtrs()->get_options('rtrs_woocommerce_settings', 'identifier');  
                        if ( $identifier_type && $identifier ) {
                            $markup[$identifier_type] = esc_html( $identifier );
                        }

                        $price = edd_get_download_price( $post->ID, false );
                        $price_valid_until = gmdate( 'Y-12-31', time() + YEAR_IN_SECONDS );
                        $markup['offers'] = array(
                            '@type'             => 'Offer',
                            'url'               => $permalink,
                            'price'             => $price,
                            'priceValidUntil'   => $price_valid_until,
                            'priceCurrency'     => 'USD',
                            'itemCondition'     => 'https://schema.org/NewCondition',
                            'availability'      => 'https://schema.org/InStock',
                        );

                        $download_rating = true;
                        if ( $download_rating ) {
                            $total_rating = Review::getTotalRatings( $post->ID );
                            if ( $total_rating ) {
                                $markup['aggregateRating'] = array(
                                    '@type'       => 'AggregateRating',
                                    'ratingValue' => Review::getAvgRatings( $post->ID ),
                                    'reviewCount' => $total_rating,
                                );
                            }
                
                            // Markup 5 most recent rating/review.
                            $comments = get_comments(
                                array(
                                    'number'      => 5,
                                    'post_id'     => $post->ID,
                                    'status'      => 'approve',
                                    'post_status' => 'publish',
                                    'post_type'   => 'download',
                                    'parent'      => 0,
                                    'meta_query'  => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
                                        array(
                                            'key'     => 'rating',
                                            'type'    => 'NUMERIC',
                                            'compare' => '>',
                                            'value'   => 0,
                                        ),
                                    ),
                                )
                            );
                
                            if ( $comments ) {
                                $markup['review'] = array();
                                foreach ( $comments as $comment ) {
                                    $markup['review'][] = array(
                                        '@type'         => 'Review',
                                        'reviewRating'  => array(
                                            '@type'       => 'Rating',
                                            'bestRating'  => '5',
                                            'ratingValue' => get_comment_meta( $comment->comment_ID, 'rating', true ),
                                            'worstRating' => '1',
                                        ),
                                        'author'        => array(
                                            '@type' => 'Person',
                                            'name'  => get_comment_author( $comment ),
                                        ),
                                        'reviewBody'    => get_comment_text( $comment ),
                                        'datePublished' => get_comment_date( 'c', $comment ),
                                    );
                                }
                            }
                        }
                    }

                    //if wc
                    global $product;
                    if ( is_a( $product, 'WC_Product' ) ) {
                
                        $shop_name = get_bloginfo( 'name' );
                        $shop_url  = home_url();
                        $currency  = get_woocommerce_currency();
                        $permalink = get_permalink( $product->get_id() );
                        
                        $markup = array(
                            "@context" => "https://schema.org",
                            '@type'       => 'Product',
                            '@id'         => $permalink . '#product', // Append '#product' to differentiate between this @id and the @id generated for the Breadcrumblist.
                            'name'        => wp_kses_post( $product->get_name() ),
                            'url'         => $permalink,
                            'description' => wp_strip_all_tags( do_shortcode( $product->get_short_description() ? $product->get_short_description() : $product->get_description() ) ),
                        );
                        
                        $image     = wp_get_attachment_url( $product->get_image_id() );
                        if ( $image ) {
                            $markup['image'] = $image;
                        }
                
                        // Declare SKU or fallback to ID.
                        if ( $product->get_sku() ) {
                            $markup['sku'] = $product->get_sku();
                        } else {
                            $markup['sku'] = $product->get_id();
                        }

                        $brand_name = rtrs()->get_options('rtrs_woocommerce_settings', 'brand_name');  
                        if ( $brand_name ) {
                            $markup['brand'] = [
                                "@type" => "Brand",
                                "name" => esc_html( $brand_name )
                            ];
                        }

                        $identifier_type = rtrs()->get_options('rtrs_woocommerce_settings', 'identifier_type');  
                        $identifier = rtrs()->get_options('rtrs_woocommerce_settings', 'identifier');  
                        if ( $identifier_type && $identifier ) {
                            $markup[$identifier_type] = esc_html( $identifier );
                        }
                
                        if ( '' !== $product->get_price() ) {
                            // Assume prices will be valid until the end of next year, unless on sale and there is an end date.
                            $price_valid_until = gmdate( 'Y-12-31', time() + YEAR_IN_SECONDS );
                
                            if ( $product->is_type( 'variable' ) ) {
                                $lowest  = $product->get_variation_price( 'min', false );
                                $highest = $product->get_variation_price( 'max', false );
                
                                if ( $lowest === $highest ) {
                                    $markup_offer = array(
                                        '@type'              => 'Offer',
                                        'price'              => wc_format_decimal( $lowest, wc_get_price_decimals() ),
                                        'priceValidUntil'    => $price_valid_until,
                                        'priceSpecification' => array(
                                            'price'                 => wc_format_decimal( $lowest, wc_get_price_decimals() ),
                                            'priceCurrency'         => $currency,
                                            'valueAddedTaxIncluded' => wc_prices_include_tax() ? 'true' : 'false',
                                        ),
                                    );
                                } else {
                                    $markup_offer = array(
                                        '@type'      => 'AggregateOffer',
                                        'lowPrice'   => wc_format_decimal( $lowest, wc_get_price_decimals() ),
                                        'highPrice'  => wc_format_decimal( $highest, wc_get_price_decimals() ),
                                        'offerCount' => count( $product->get_children() ),
                                    );
                                }
                            } else {
                                if ( $product->is_on_sale() && $product->get_date_on_sale_to() ) {
                                    $price_valid_until = gmdate( 'Y-m-d', $product->get_date_on_sale_to()->getTimestamp() );
                                }
                                $markup_offer = array(
                                    '@type'              => 'Offer',
                                    'price'              => wc_format_decimal( $product->get_price(), wc_get_price_decimals() ),
                                    'priceValidUntil'    => $price_valid_until,
                                    'priceSpecification' => array(
                                        'price'                 => wc_format_decimal( $product->get_price(), wc_get_price_decimals() ),
                                        'priceCurrency'         => $currency,
                                        'valueAddedTaxIncluded' => wc_prices_include_tax() ? 'true' : 'false',
                                    ),
                                );
                            }
                
                            $markup_offer += array(
                                'priceCurrency' => $currency,
                                'availability'  => 'https://schema.org/' . ( $product->is_in_stock() ? 'InStock' : 'OutOfStock' ),
                                'url'       => $permalink,
                                'seller'    => array(
                                    '@type' => 'Organization',
                                    'name'  => $shop_name,
                                    'url'   => $shop_url,
                                ),
                            );
                
                            $markup['offers'] = array( apply_filters( 'woocommerce_structured_data_product_offer', $markup_offer, $product ) );
                        }
                
                        if ( $product->get_rating_count() && wc_review_ratings_enabled() ) {
                            $markup['aggregateRating'] = array(
                                '@type'       => 'AggregateRating',
                                'ratingValue' => $product->get_average_rating(),
                                'reviewCount' => $product->get_review_count(),
                            );
                
                            // Markup 5 most recent rating/review.
                            $comments = get_comments(
                                array(
                                    'number'      => 5,
                                    'post_id'     => $product->get_id(),
                                    'status'      => 'approve',
                                    'post_status' => 'publish',
                                    'post_type'   => 'product',
                                    'parent'      => 0,
                                    'meta_query'  => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
                                        array(
                                            'key'     => 'rating',
                                            'type'    => 'NUMERIC',
                                            'compare' => '>',
                                            'value'   => 0,
                                        ),
                                    ),
                                )
                            );
                
                            if ( $comments ) {
                                $markup['review'] = array();
                                foreach ( $comments as $comment ) {
                                    $markup['review'][] = array(
                                        '@type'         => 'Review',
                                        'reviewRating'  => array(
                                            '@type'       => 'Rating',
                                            'bestRating'  => '5',
                                            'ratingValue' => get_comment_meta( $comment->comment_ID, 'rating', true ),
                                            'worstRating' => '1',
                                        ),
                                        'author'        => array(
                                            '@type' => 'Person',
                                            'name'  => get_comment_author( $comment ),
                                        ),
                                        'reviewBody'    => get_comment_text( $comment ),
                                        'datePublished' => get_comment_date( 'c', $comment ),
                                    );
                                }
                            }
                        }
                    }

                    if ( $markup ) {
                        $html .= $this->getJsonEncode(apply_filters('rtseo_snippet_product', $markup, $metaData));
                    }
                    break;
                
                default:
                    break;
            }
        }
        return $html; 
    }
    
}